/**
 * $Id: configuration.cpp 167 2010-01-29 17:00:13Z l0728348 $
 *
 * Copyright 2009
 *
 * @author Christian, Bernhard, Manuel / Gruppe 1
 * @brief  handles commandline and configuration file parsing
 */

#include <iostream>
#include <vector>
#include <fstream>
#include <boost/foreach.hpp>
#include <boost/shared_ptr.hpp>
#include <libgen.h>
#include <stdlib.h>
#include <string.h>
#include "logger.h"
#include "configuration.h"
#include "security.h"

/* some default values */
#define DEF_NET_ENDPOINT "ssl -h localhost -p 3434"
#define DEF_DB_TYPE      "postgresql"
#define DEF_DB_POOLSIZE  1
#define DEF_MISC_ACL     "allow"

#ifdef HAVE_CONFIG_H
# include "config.h"
#else
# define SYSCONFDIR "conf"
# define PACKAGE    "ehrprovider"
#endif

using namespace std;
using namespace xmlpp;
namespace po = boost::program_options;

namespace
{
  static log4cxx::LoggerPtr logger(log4cxx::Logger::getLogger("Config"));
}

/*----------------------------------------------------------------------------*/

Configuration::Configuration(int argc, char *argv[])
throw()
  : m_shouldexit(false), m_configfile(""), m_configdir(""), m_myname(""),
  m_lasterror("")
{
  if (!log4cxx::Logger::getRootLogger()->getAllAppenders().empty())
    LOG4CXX_TRACE(logger, "Configuration()");

  typedef vector<string> string_vector;

  /* define commandline options */
  po::options_description cmdline_options("General options");
  cmdline_options.add_options()
    ("help,h",    "this help message")
    ("version,v", "output the version number")
    ("D,D",       "detach from foreground")
    ("d,d",       "enable debug mode")
    ("t,t",       "enable trace mode")
    ("e,e",       "send log messages to stderr")
    ("o,o",       po::value<string_vector>(), "override ini setting via commandline. Syntax: ini.full.path.key=value")
    ("f,f",       po::value<string>(), "specify configurationfile")
    ("Ice.*",     po::value<string>(), "Ice commandline options")
    ("IceSSL.*",  po::value<string>(), "IceSSL commandline options");

  /* define allowed ini settings */
  po::options_description config_options("Configuration");
  config_options.add_options()
    ("network.endpoint",         po::value<string>()->default_value(DEF_NET_ENDPOINT))
    ("database.type",            po::value<string>()->default_value(DEF_DB_TYPE))
    ("database.dsn",             po::value<string>())
    ("database.poolsize",        po::value<unsigned>()->default_value(DEF_DB_POOLSIZE))
    ("misc.defaultacl",          po::value<string>()->default_value(DEF_MISC_ACL))
    ("certificates.rootca",      po::value<string>())
    ("certificates.mycert",      po::value<string>())
    ("certificates.mykey",       po::value<string>())
    ("certificates.users",       po::value<string>())
    ("certificates.providers",   po::value<string>())
    ("certificates.providermap", po::value<string>())
    ("log4j.*",                  po::value<string>())
    ("Ice.*",                    po::value<string>())
    ("IceSSL.*",                 po::value<string>());

  /* parse commandline options */
  try
  {
    po::store(po::command_line_parser(argc, argv).options(cmdline_options).run(), m_cmdvm);
    po::notify(m_cmdvm);
  }
  catch(po::error& ex)
  {
    m_lasterror << "Error while parsing commandline option: " << ex.what();
    return;
  }

  if (m_cmdvm.count("help"))
  {
    cout << PACKAGE_STRING << endl
         << "Usage: " << argv[0] << " [options]" << endl
         << cmdline_options << endl;
    m_shouldexit = true;
    return;
  }

  if (m_cmdvm.count("version"))
  {
    cout << VERSION << endl;
    m_shouldexit = true;
    return;
  }

  /* create and store configfile and configdir */
  m_configfile = (m_cmdvm.count("f")) ? m_cmdvm["f"].as<string>() : SYSCONFDIR "/" PACKAGE ".conf";
  char *conf = strdup(m_configfile.c_str());
  m_configdir = dirname(conf);
  free(conf);

  /* open ini file */
  ifstream ifs(m_configfile.c_str());
  if (!ifs)
  {
    m_lasterror << "Unable to open configuration file: \"" << m_configfile << "\"";
    return;
  }

  /* parse ini file */
  try
  {
    po::store(parse_config_file(ifs, config_options), m_cfgvm);
    po::notify(m_cfgvm);
  }
  catch(po::error& ex)
  {
    m_lasterror << "Error while parsing configuration file: " << ex.what();
    ifs.close();
    return;
  }

  /* there's no exception if configuration file is a directory */
  if (ifs.bad())
  {
    m_lasterror << "Unable to read from configuration file";
    ifs.close();
    return;
  }
  ifs.close();

  /*
   * override ini settings passed via commandline
   * use vm::iterator as operator[] returns const value
   */
  if (m_cmdvm.count("o"))
  {
    BOOST_FOREACH(string param, m_cmdvm["o"].as<string_vector>())
    {
      size_t pos = param.find_first_of('=');
      if (pos == string::npos)
      {
        m_lasterror << "Error while parsing ini settings from commandline. Invalid syntax";
        return;
      }

      string cmdkey   = param.substr(0, pos);
      string cmdvalue = param.substr(pos + 1);
      if (cmdkey.empty() || cmdvalue.empty())
      {
        m_lasterror << "Error while parsing ini settings from commandline. Either key or value is empty";
        return;
      }

      const po::option_description *option = config_options.find_nothrow(cmdkey, false);
      if (!option)
      {
        m_lasterror << "Error while parsing ini settings from commandline. Unknown key \"" << cmdkey << "\"";
        return;
      }

      /* this kinda hack, but works */
      std::map<std::string, po::variable_value>& vmap = m_cfgvm;
      po::variable_value& optvalue = vmap[cmdkey];
      /* explicit assignment here erases defaulted value */
      if (optvalue.defaulted())
        optvalue = po::variable_value();

      /* put cmdvalue into a vector */
      vector<string> tmpvec;
      tmpvec.push_back(cmdvalue);

      /* avoid multiple_occurrences-exception so erase the value */
      optvalue.value() = boost::any();
      try
      {
        option->semantic()->parse(optvalue.value(), tmpvec, false);
      }
      catch(po::error& ex)
      {
        m_lasterror << "Error while parsing ini settings from commandline: " << ex.what();
        return;
      }
    }
  }
}

/*----------------------------------------------------------------------------*/

void Configuration::checkCertificates()
throw(runtime_error)
{
  LOG4CXX_TRACE(logger, "checkCertificates()");
  LOG4CXX_DEBUG(logger, "Checking Certificates");

  /* check required configuration settings  */
  const char *conf[] = { "certificates.rootca", "certificates.mycert", "certificates.mykey" };
  BOOST_FOREACH(const char *var, conf)
  {
    if (!hasVal(var))
      throw runtime_error("Setting \"" + string(var) + "\" is undefined. This is required!");
    string certfile = getConfPath(val<string>(var));
    ifstream ifs(certfile.c_str());
    if (!ifs)
      throw runtime_error("Unable to open certificate file: \"" + certfile + "\"");
    ifs.close();
  }

  /* load my own commonname */
  LOG4CXX_DEBUG(logger, "Extracting commonname from mycert");
  string certfile = getConfPath(val<string>("certificates.mycert"));
  string commonname;
  try
  {
    commonname = Security::instance().getCommonName(certfile);
  }
  catch(SecurityException& ex)
  {
    throw runtime_error("Unable to load certificate file: \"" + certfile + "\":" + ex.what());
  }
  size_t pos = commonname.find_first_of('@');
  m_myname = commonname.substr(0, pos);
}

/*----------------------------------------------------------------------------*/

void Configuration::loadProviderMap()
throw(runtime_error)
{
  LOG4CXX_TRACE(logger, "loadProviderMap()");
  LOG4CXX_DEBUG(logger, "Checking ProviderMap");

  /* check required configuration settings  */
  const char *conf[] = { "certificates.users", "certificates.providers", "certificates.providermap" };
  BOOST_FOREACH(const char *var, conf)
  {
    if (!hasVal(var))
      throw runtime_error("Setting \"" + string(var) + "\" is undefined. This is required!");
  }
  string providermapfile = getConfPath(val<string>("certificates.providermap"));
  string providersdir = getConfPath(val<string>("certificates.providers"));

  /* parse xml */
  LOG4CXX_DEBUG(logger, "Parsing ProviderMap-File: " << providermapfile);
  try
  {
    m_domparser.set_validate(true);
    m_domparser.set_substitute_entities(true);
    m_domparser.parse_file(providermapfile);
    if (!m_domparser)
      throw runtime_error("Unable to parse ProviderMap-File. Unknown error");

    const Element *root = m_domparser.get_document()->get_root_node();
    if (!root)
      throw runtime_error("Unable to parse ProviderMap-File: No root node");

    BOOST_FOREACH(Node *tmp, root->find("/providers/provider/cert/text()"))
    {
      const TextNode *tnode = dynamic_cast<const TextNode *>(tmp);
      string certfile = providersdir + "/" + tnode->get_content();
      ifstream ifs(certfile.c_str());
      if (!ifs)
        throw runtime_error("Unable to open provider certificate file: \"" + certfile + "\"");
      ifs.close();
    }
  }
  catch(xmlpp::exception &ex)
  {
    throw runtime_error("Unable to parse ProviderMap-File: " + string(ex.what()));
  }
}

/*----------------------------------------------------------------------------*/

bool Configuration::loadCertificates()
throw()
{
  try
  {
    checkCertificates();
    loadProviderMap();
  }
  catch(runtime_error &ex)
  {
    LOG4CXX_FATAL(logger, ex.what());
    return false;
  }
  return true;
}

/*----------------------------------------------------------------------------*/

const std::string Configuration::getUserCert(const std::string& user, const std::string& provider)
throw()
{
  LOG4CXX_TRACE(logger, "getUserCert(\"" << user << "\", \"" + provider + "\")");

  stringstream ss;
  ss << getConfPath(val<string>("certificates.users")) << "/"
     << user << "@" << provider << ".pem";

  string file = ss.str();
  ifstream ifs(file.c_str());
  if (!ifs)
  {
    LOG4CXX_ERROR(logger, "Unable to open certificate file: \"" << file << "\"");
    return "";
  }
  return file;
}

/*----------------------------------------------------------------------------*/

const std::string Configuration::getProviderNodeText(const std::string& provider, const std::string& node)
throw()
{
  IceUtil::Mutex::Lock lock(m_mutex);
  LOG4CXX_TRACE(logger, "getProviderNodeText(\"" << provider << "\", \"" + node + "\")");

  try
  {
    const Element *root = m_domparser.get_document()->get_root_node();
    NodeSet nodes = root->find("/providers/provider[@id=\"" + provider + "\"]/" + node + "/text()");
    if (nodes.size() == 0)
      throw runtime_error("Provider \"" + provider + "\" is not known");
    else if(nodes.size() > 1)
      throw runtime_error("Provider \"" + provider + "\" is defined more than once");

    const TextNode *tnode = dynamic_cast<const TextNode *>(nodes[0]);
    return tnode->get_content();
  }
  catch(xmlpp::exception &ex)
  {
    LOG4CXX_ERROR(logger, "Unable to parse ProviderMap: " << ex.what());
  }
  catch(runtime_error &ex)
  {
    LOG4CXX_ERROR(logger, "Unable to parse ProviderMap: " << ex.what());
  }
  return "";
}

/* vim: set et sw=2 ts=2: */
