/**
 * $Id: configuration.h 145 2010-01-21 00:34:26Z l0728348 $
 *
 * Copyright 2009
 *
 * @author Christian, Bernhard, Manuel / Gruppe 1
 * @brief  handles commandline and configuration file parsing
 */

#ifndef CONFIGURATION_H
#define CONFIGURATION_H

#include <stdexcept>
#include <string>
#include <sstream>
#include <boost/program_options.hpp>
#include <boost/lexical_cast.hpp>
#include <libxml++/libxml++.h>
#include <IceUtil/Mutex.h>

/**
 * handles commandline and configuration file parsing
 * all methods only throw Configuration::runtime_error exceptions!
 */
class Configuration
{
  public:
    /**
     * exception thrown by Configuration
     */
    class runtime_error
      : public std::runtime_error
    {
      public:
        /**
         * Default exception ctor
         * @param  what message to pass along
         */
        runtime_error(const std::string& what)
          : std::runtime_error(what)
        {}
    };

    /**
     * default ctor
     * sets up commandline options and parses them
     * also parses configuration file and handles support for
     * overriding configuration file settings via commandline
     *
     * if application should exit gracefully (e.g. after printing usage)
     * method terminate() will return true
     * on error operator!() returns false and getLastError is set
     *
     * @param  argc  argc from main
     * @param  argv  argv from main
     */
    Configuration(int argc, char *argv[])
      throw();

    /**
     * default dtor
     */
    ~Configuration()
      throw()
    {}

    /**
     * get value for configuration setting.
     * commandline settings precede configuration file settings.
     * @param  key  key of configuration setting
     * @return value of configuration setting
     * @throw  runtime_error if key is not found or setting is not of type T
     */
    template<typename T>
    const T& val(const std::string& key)
      throw(runtime_error)
    {
      const boost::program_options::variable_value &vval = (!m_cmdvm[key].empty()) ? m_cmdvm[key] : m_cfgvm[key];
      if (vval.empty())
        throw runtime_error("Key not found");
      try
      {
        return vval.as<T>();
      }
      catch(boost::bad_any_cast& ex)
      {
        throw runtime_error("Unable to convert value");
      }
    }

    /**
     * checks if a configuration setting exists/is set
     * (either from commandline or from configuration file)
     * @param  key  key of configuration setting
     * @return true if setting exist. false otherwise
     */
    const bool hasVal(const std::string& key)
      throw()
    {
      if (m_cmdvm[key].empty() && m_cfgvm[key].empty())
        return false;
      return true;
    }

    /**
     * get error message from last error
     * @return error message from last error
     */
    const std::string getLastError()
    {
      return m_lasterror.str();
    }

    /**
     * check if an error occured during parsing
     * @return true on error, false otherwise
     */
    const bool operator!()
    {
      return !m_lasterror.str().empty();
    }

    /**
     * check whether application should exit gracefully
     * @return true if application should exit gracefully, false otherwise
     */
    const bool shouldExit()
    {
      return m_shouldexit;
    }

    /**
     * get path to configuration file
     * @return path to configuration file
     */
    const std::string& getConfFile()
    {
      return m_configfile;
    }

    /**
     * get path to configuration directory
     * @return path to configuration directory
     */
    const std::string& getConfDir()
    {
      return m_configdir;
    }

    /**
     * get path to file inside configuration directory
     * @param  path relative or absolut path to file
     * @return path to file inside configuration directory
     */
    const std::string getConfPath(const std::string& path)
    {
      if (path.empty() || path[0] == '/')
        return path;
      return getConfDir() + '/' + path;
    }

    /**
     * checks and loads certificates in configuration file
     * (.e.g providermap, rootca, etc..)
     * @return true on success, false otherwise
     */
    bool loadCertificates()
      throw();

    /**
     * retrieve path to certificate of specific user (read from providermap)
     * @param  user username belonging to certificate
     * @return path to user certificate or empty string on error
     */
    const std::string getUserCert(const std::string& user)
    {
      return getUserCert(user, getMyName());
    }

    /**
     * retrieve path to certificate of specific user at specific provider
     * (read from providermap)
     * @param  user     username belonging to certificate
     * @param  provider provider belonging to certificate
     * @return path to user certificate or empty string on error
     */
    const std::string getUserCert(const std::string& user, const std::string& provider)
      throw();

    /**
     * retrieve path to certificate of specific provider (read from providermap)
     * @param  provider name of provider
     * @return path to provider certificate or empty string on error
     */
    const std::string getProviderCert(const std::string& provider)
    {
      /* this certificate must exist. already checked in loadCertificates() */
      std::string tmp = getProviderNodeText(provider, "cert");
      return (!tmp.empty()) ? getConfPath(val<std::string>("certificates.providers")) + "/" + tmp : "";
    }

    /**
     * retrieve endpoint of specific provider (read from providermap)
     * @param  provider name of provider
     * @return path to provider certificate or empty string on error
     */
    const std::string getProviderEndpoint(const std::string& provider)
    {
      return getProviderNodeText(provider, "endpoint");
    }

    /**
     * get provider name of current instance
     * @return provider name of current instance
     */
    const std::string& getMyName()
    {
      return m_myname;
    }

  protected:
    /**
     * checks certificates referenced in configuration file.
     * also parses commonname from own certificate for getMyName()
     * @throw  runtime_error on error
     */
    void checkCertificates()
      throw(runtime_error);

    /**
     * loads, validates and parses providermap file.
     * also checks all referenced certificates
     * @throw  runtime_error on error
     */
    void loadProviderMap()
      throw(runtime_error);

    /**
     * retrieve content of a specific childnode of
     * a specific provider node in providermap
     * @param  provider name of provider-node
     * @param  node name of childnode inside provider-node
     * @return content in xmlnode
     */
    const std::string getProviderNodeText(const std::string& provider, const std::string& node)
      throw();

  private:
    bool m_shouldexit;
    std::string m_configfile;
    std::string m_configdir;
    std::string m_myname;
    std::stringstream m_lasterror;
    boost::program_options::variables_map m_cmdvm;
    boost::program_options::variables_map m_cfgvm;
    xmlpp::DomParser m_domparser;
    IceUtil::Mutex m_mutex;
};

#endif

/* vim: set et sw=2 ts=2: */
