/**
 * $Id: connectionpool.cpp 169 2010-01-29 17:03:03Z l0728348 $
 *
 * Copyright 2009
 *
 * @author Christian, Bernhard, Manuel / Gruppe 1
 * @brief Database Connection Pool
 */

#include "connectionpool.h"
#include "logger.h"
#include <pqxx/pqxx>

using namespace pqxx;

namespace
{
  static log4cxx::LoggerPtr logger(log4cxx::Logger::getLogger("ConnectionPool"));
}

/*----------------------------------------------------------------------------*/

ConnectionPool::ConnectionPool(const std::string& info, array_type::size_type size)
{
  LOG4CXX_TRACE(logger, "ConnectionPool()");

  LOG4CXX_DEBUG(logger, "Initializing Connection pool with " << size << " connections");
  m_free = 0;
  m_pool.reserve(size);
  m_inuse.resize(size, false);
  for (array_type::size_type i = 0; i < size; ++i)
  {
    connection *conn = (info == "DEFAULT") ? new connection() : new connection(info);
    m_pool.push_back(value_type(conn));
  }
  m_free = size;

  LOG4CXX_INFO(logger, "Connection pool initialized");
}

/*----------------------------------------------------------------------------*/

ConnectionPool::~ConnectionPool()
{
  LOG4CXX_TRACE(logger, "~ConnectionPool()");
  while(!m_pool.empty())
  {
    m_pool.back()->disconnect();
    m_pool.pop_back();
  }
  m_free = 0;
}

/*----------------------------------------------------------------------------*/

const ConnectionPool::Connection ConnectionPool::get()
{
  LOG4CXX_TRACE(logger, "get()");
  IceUtil::Monitor<IceUtil::Mutex>::Lock lock(*this);

  while (m_free == 0)
  {
    LOG4CXX_WARN(logger, "No more database connections available. You should increase database connection pool size");
    LOG4CXX_DEBUG(logger, "Waiting for connection...");
    wait();
  }

  for (array_type::size_type i = 0; i < m_inuse.size(); ++i )
  {
    if (!m_inuse[i])
    {
      LOG4CXX_DEBUG(logger, "Requested free connection. Assigned connection #" << i);
      m_inuse[i] = true;
      --m_free;
      return Connection(this, m_pool[i], m_inuse[i]);
    }
  }
}

/*----------------------------------------------------------------------------*/

void ConnectionPool::release(bitmap_type::reference inuse)
{
  LOG4CXX_TRACE(logger, "release()");
  IceUtil::Monitor<IceUtil::Mutex>::Lock lock(*this);
  inuse = false;
  m_free++;
  notifyAll();
}

/* vim: set et sw=2 ts=2: */
