/**
 * $Id: connectionpool.h 169 2010-01-29 17:03:03Z l0728348 $
 *
 * Copyright 2009
 *
 * @author Christian, Bernhard, Manuel / Gruppe 1
 * @brief Database Connection Pool
 */

#ifndef CONNECTIONPOOL_H
#define CONNECTIONPOOL_H

#include <string>
#include <vector>
#include <boost/shared_ptr.hpp>
#include <pqxx/connection>
#include <Ice/Ice.h>
#include <IceUtil/Mutex.h>
#include <IceUtil/Monitor.h>

/**
 * Database connection pool implementation
 * Initializes a set of connections available for concurrent database usage
 */
class ConnectionPool
  : public IceUtil::Monitor<IceUtil::Mutex>
{
  protected:
    /** type of connections inside the pool */
    typedef boost::shared_ptr<pqxx::connection> value_type;
    /** array type for connections */
    typedef std::vector<value_type> array_type;
    /** bitmap type for marking connections as "used" or "free" */
    typedef std::vector<bool> bitmap_type;

  public:
    /**
     * Connection from database connection pool
     * Connection releases itself upon destructions
     */
    class Connection
    {
      public:
        /**
         * default ctor
         * @param  pool pointer to the ConnectionPool instance
         * @param  value reference of the internal connection
         * @param  inuse reference of the use-bitmap
         */
        Connection(ConnectionPool *pool, array_type::reference value,
            bitmap_type::reference inuse)
          : m_pool(pool), m_value(value), m_inuse(inuse)
        {}

        /**
         * default dtor
         * releases this connection
         */
        ~Connection()
        {
          m_pool->release(m_inuse);
        }

        /**
         * reference of the internal connection
         * @return reference of the internal connection
         */
        const array_type::reference get()
        {
          return m_value;
        }

        /**
         * conversion operator for quickly get a pointer to the internal connection
         * @return pointer to the internal connection
         */
        operator pqxx::connection*() const
        {
          return m_value.get();
        }

        /**
         * pointer operator for quickly get a pointer to the internal connection
         * @return pointer to the internal connection
         */
        pqxx::connection* operator->() const
        {
          return m_value.get();
        }

      private:
        ConnectionPool *m_pool;
        array_type::reference m_value;
        bitmap_type::reference m_inuse;
    };

    /**
     * default ctor
     * @param  info connection string
     * @param  size size of connection pool
     * @throw  PGSTD::runtime_error upon error
     */
    ConnectionPool(const std::string& info, array_type::size_type size);

    /**
     * default dtor
     * destroys all connections
     */
    ~ConnectionPool();

    /**
     * retrieve an assigned connection from the pool. the connection
     * won't be assigned again until released
     * @return assigned connection from pool
     */
    const Connection get();

  protected:
    /**
     * release connection so it can be assigned again
     */
    void release(bitmap_type::reference inuse);

  private:
    array_type m_pool;
    bitmap_type m_inuse;
    unsigned m_free;
};

#endif

/* vim: set et sw=2 ts=2: */
