/**
 * $Id: datamanager.cpp 96 2010-01-18 01:22:43Z l0728348 $
 *
 * Copyright 2009
 *
 * @author Christian, Bernhard, Manuel / Gruppe 1
 * @brief Interface for accessing data
 */

#include "security.h"
#include "datamanager.h"
#include "logger.h"
#include <sstream>
#include <boost/foreach.hpp>
#include <pqxx/pqxx>

using namespace std;
using namespace Ehr;
using namespace pqxx;

namespace
{
  static log4cxx::LoggerPtr logger(log4cxx::Logger::getLogger("DataManager"));
}

/*----------------------------------------------------------------------------*/

DataManager::DataManager(Configuration *conf, Ice::CommunicatorPtr comm)
  : m_lasterror(""), m_conf(conf), m_pool(NULL), m_comm(comm)
{
  LOG4CXX_TRACE(logger, "DataManager()");
  if (m_conf == NULL)
  {
    m_lasterror << "Internal error: Configuration object is null.";
    return;
  }

  /* check required configuration settings  */
  const char *confs[] = { "database.type", "database.dsn" };
  BOOST_FOREACH(const char *var, confs)
  {
    if (!m_conf->hasVal(var))
    {
      m_lasterror << "Setting \"" << string(var) << "\" is undefined. This is required!";
      return;
    }
  }

  /* check supported database type */
  string dbtype = m_conf->val<string>("database.type");
  if (dbtype != "postgresql")
  {
    m_lasterror << "Unsupported Database-Type \"" << dbtype << "\".";
    return;
  }

  unsigned poolsize = m_conf->val<unsigned>("database.poolsize");
  if (poolsize <= 0)
  {
    m_lasterror << "Database connection pool size must be greater than zero.";
    return;
  }
}

/*----------------------------------------------------------------------------*/

bool DataManager::connect()
{
  LOG4CXX_TRACE(logger, "Connect()");
  if (m_pool != NULL)
    return true;

  try
  {
    m_pool = new ConnectionPool(m_conf->val<string>("database.dsn"), m_conf->val<unsigned>("database.poolsize"));
  }
  catch(PGSTD::runtime_error &ex)
  {
    m_pool = NULL;
    m_lasterror << "Unable to connect to database!" << std::endl << ex.what();
    LOG4CXX_FATAL(logger, m_lasterror.str());
    return false;
  }

  LOG4CXX_INFO(logger, "Connected to database");
  return true;
}

/*----------------------------------------------------------------------------*/

bool DataManager::disconnect()
{
  LOG4CXX_TRACE(logger, "Disconnect()");
  if (m_pool == NULL)
    return true;

  delete m_pool;
  m_pool = NULL;

  LOG4CXX_INFO(logger, "Disconnected from database");
  return true;
}

/*----------------------------------------------------------------------------*/

bool DataManager::checkSignature(const std::string& user,
    const Ehr::Signature& signature, Ehr::RequestPtr req)
throw(Ehr::EhrException)
{
  LOG4CXX_TRACE(logger, "checkSignature(\"" << user << "\", ...)");

  string usercert = m_conf->getUserCert(user);
  if (usercert.empty())
  {
    LOG4CXX_WARN(logger, "Certificate for user \"" << user << "\" doesn't exist");
    throw InvalidAccountException("Unknown user account");
  }

  try
  {
    IceUtil::Mutex::Lock lock(m_mutex);
    const vector<Ice::Byte> data = convertToByteStream(req);
    Security::instance().verifySignature(usercert, data, signature.data);
  }
  catch(SecurityException &ex)
  {
    LOG4CXX_WARN(logger, "Unable to verify signature of user \"" << user << "\": " << ex.what());
    return false;
  }

  return true;
}

/*----------------------------------------------------------------------------*/

bool DataManager::checkProviderSignature(const std::string& provider,
    const Ehr::Signature& signature, Ehr::RequestPtr req)
throw(Ehr::EhrException)
{
  LOG4CXX_TRACE(logger, "checkProviderSignature(\"" << provider << "\", ...)");

  string providercert = m_conf->getProviderCert(provider);
  if (providercert.empty())
  {
    LOG4CXX_WARN(logger, "Certificate for provider \"" << provider << "\" doesn't exist");
    throw InvalidAccountException("Unknown provider account");
  }

  try
  {
    IceUtil::Mutex::Lock lock(m_mutex);
    const vector<Ice::Byte> data = convertToByteStream(req);
    Security::instance().verifySignature(providercert, data, signature.data);
  }
  catch(SecurityException &ex)
  {
    LOG4CXX_WARN(logger, "Unable to sign signature of provider \"" << provider << "\": " << ex.what());
    return false;
  }

  return true;
}
/*----------------------------------------------------------------------------*/

void DataManager::signProviderRequest(const Ehr::RequestPtr& req,
    Ehr::Signature& signature)
throw(runtime_error)
{
  LOG4CXX_TRACE(logger, "signProviderRequest(...)");

  string mykey = m_conf->getConfPath(m_conf->val<string>("certificates.mykey"));
  if (mykey.empty())
  {
    LOG4CXX_ERROR(logger, "Certificatekey of myself got lost!");
    throw runtime_error("Unable to sign my own request");
  }

  try
  {
    IceUtil::Mutex::Lock lock(m_mutex);
    const vector<Ice::Byte> data = convertToByteStream(req);
    Security::instance().sign(mykey, data, signature.data);
  }
  catch(SecurityException &ex)
  {
    LOG4CXX_WARN(logger, "Unable to sign provider request: " << ex.what());
    throw runtime_error("Unable to sign my own request");
  }
}

/*----------------------------------------------------------------------------*/

void DataManager::createPrescription(const Ehr::EncryptedDocument& doc,
    const Ehr::AccountIdentifier& owner)
throw(runtime_error, Ehr::EhrException)
{
  LOG4CXX_TRACE(logger, "createPrescription(doc, \"" << owner.user << "@"
      << owner.provider << "\")");

  try
  {
    ConnectionPool::Connection conn = getConnection();

    /* create a new document */
    work T(*conn, "createPrescription");
    conn->prepare("createPrescription",
        "INSERT INTO documents "
        "VALUES (DEFAULT, (SELECT id FROM users WHERE name = $1), "
        "  'PRESCRIPTION', DEFAULT, $2, TRUE, $3, $4, $5)")
      ("TEXT",    prepare::treat_string)
      ("VARCHAR", prepare::treat_string)
      ("BYTEA",   prepare::treat_binary)
      ("BYTEA",   prepare::treat_binary)
      ("BYTEA",   prepare::treat_binary);
    T.prepared("createPrescription")
      (owner.user)
      ((m_conf->val<std::string>("misc.defaultacl") == "allow") ? "ALLOW" : "DENY")
      (string(doc.encryptedKey.begin(),  doc.encryptedKey.end()))
      (string(doc.initialVector.begin(), doc.initialVector.end()))
      (string(doc.encryptedData.begin(), doc.encryptedData.end())).exec();
    T.commit();
  }
  catch(PGSTD::runtime_error &ex)
  {
    LOG4CXX_ERROR(logger, "Unable to execute query: " << ex.what());
    throw runtime_error("Unable to execute query");
  }
}

/*----------------------------------------------------------------------------*/

void DataManager::consumePrescription(const Ehr::EncryptedDocument& doc,
    Ice::Long prescriptionToConsume, const Ehr::AccountIdentifier& owner,
    const Ehr::AccountIdentifier& requestor)
throw(runtime_error, Ehr::EhrException)
{
  LOG4CXX_TRACE(logger, "consumePrescription(doc, " << prescriptionToConsume
      << ", \"" << owner.user << "@" << owner.provider << "\", \""
      << requestor.user << "@" << requestor.provider << "\")");

  try
  {
    ConnectionPool::Connection conn = getConnection();

    /* disable the old document - but only if allowed */
    work T1(*conn, "disableDocument");
    conn->prepare("disableDocument",
        "UPDATE documents d "
        "SET valid = FALSE "
        "FROM users u "
        "WHERE d.id = $1 AND d.userid = u.id AND u.name = $2 AND d.valid = TRUE AND (( "
        "  d.defacl = 'ALLOW' AND d.id NOT IN ( "
        "    SELECT p.docid "
        "    FROM permissions p "
        "    WHERE username = $3 AND provider = $4 AND acl = 'DENY') "
        ") OR ( "
        "  d.defacl = 'DENY' AND d.id IN ( "
        "    SELECT p.docid "
        "    FROM permissions p "
        "    WHERE username = $3 AND provider = $4 AND acl = 'ALLOW') "
        "))")
      ("INTEGER", prepare::treat_direct)
      ("TEXT",    prepare::treat_string)
      ("TEXT",    prepare::treat_string)
      ("TEXT",    prepare::treat_string);
    result res = T1.prepared("disableDocument")
      (prescriptionToConsume)
      (owner.user)
      (requestor.user)
      (requestor.provider).exec();
    T1.commit();

    if (res.affected_rows() == 0)
      throw PermissionDeniedException("Not allowed to access this document");

    /* create a new document */
    work T2(*conn, "createConsumedPrescription");
    conn->prepare("createConsumedPrescription",
        "INSERT INTO documents "
        "VALUES (DEFAULT, (SELECT id FROM users WHERE name = $1), "
        "  'CONSUMEDPRESCRIPTION', DEFAULT, $2, TRUE, $3, $4, $5)")
      ("TEXT",     prepare::treat_string)
      ("VARCHAR", prepare::treat_string)
      ("BYTEA",   prepare::treat_binary)
      ("BYTEA",   prepare::treat_binary)
      ("BYTEA",   prepare::treat_binary);
    T2.prepared("createConsumedPrescription")
      (owner.user)
      ((m_conf->val<std::string>("misc.defaultacl") == "allow") ? "ALLOW" : "DENY")
      (string(doc.encryptedKey.begin(),  doc.encryptedKey.end()))
      (string(doc.initialVector.begin(), doc.initialVector.end()))
      (string(doc.encryptedData.begin(), doc.encryptedData.end())).exec();
    T2.commit();
  }
  catch(PGSTD::runtime_error &ex)
  {
    LOG4CXX_ERROR(logger, "Unable to execute query: " << ex.what());
    throw runtime_error("Unable to execute query");
  }
}

/*----------------------------------------------------------------------------*/

boost::shared_ptr<Ehr::DocumentList> DataManager::listDocuments(
    const Ehr::AccountIdentifier& owner, const Ehr::AccountIdentifier& requestor,
    const Ehr::UserType requestorType)
throw(runtime_error, Ehr::EhrException)
{
  LOG4CXX_TRACE(logger, "listDocuments(\"" << owner.user << "@" << owner.provider
      << "\", \"" << requestor.user << "@" << requestor.provider << "\")");
  boost::shared_ptr<DocumentList> doclist(new DocumentList);

  try
  {
    ConnectionPool::Connection conn = getConnection();

    /* build dynamic WHERE-string */
    stringstream optstring("");
    if (requestorType == PHARMACIST)
      optstring << "AND d.type = 'PRESCRIPTION' ";

    /* get documentlist */
    work T(*conn, "listDocuments");
    result res;
    conn->unprepare("listDocuments");
    if (owner.user == requestor.user && owner.provider == requestor.provider)
    {
      /* documentlist of own documents */
      conn->prepare("listDocuments",
          "SELECT *, FLOOR(EXTRACT(EPOCH FROM d.created) * 1000) AS msecs "
          "FROM documents d "
          "JOIN users u ON (d.userid = u.id AND u.name = $1) "
          "WHERE d.valid = TRUE " + optstring.str())
        ("TEXT", prepare::treat_string);
      res = T.prepared("listDocuments")
        (owner.user).exec();
    }
    else
    {
      /* documentlist of other person */
      conn->prepare("listDocuments",
          "SELECT *, FLOOR(EXTRACT(EPOCH FROM d.created) * 1000) AS msecs "
          "FROM documents d "
          "JOIN users u ON (d.userid = u.id AND u.name = $1) "
          "WHERE d.valid = TRUE AND (( "
          "  d.defacl = 'ALLOW' AND d.id NOT IN ( "
          "    SELECT p.docid "
          "    FROM permissions p "
          "    WHERE username = $2 AND provider = $3 AND acl = 'DENY') "
          ") OR ( "
          "  d.defacl = 'DENY' AND d.id IN ( "
          "    SELECT p.docid "
          "    FROM permissions p "
          "    WHERE username = $2 AND provider = $3 AND acl = 'ALLOW') "
          ")) " + optstring.str())
        ("TEXT", prepare::treat_string)
        ("TEXT", prepare::treat_string)
        ("TEXT", prepare::treat_string);
      res = T.prepared("listDocuments")
        (owner.user)
        (requestor.user)
        (requestor.provider).exec();
    }
    T.commit();

    /* add documents to doclist */
    for (result::const_iterator it = res.begin(); it != res.end(); ++it)
    {
      DocumentListItem item;
      item.id = (*it)["id"].as<Ice::Long>();
      item.created.msecs = (*it)["msecs"].as<Ice::Long>();
      binarystring enckey((*it)["encKey"]);
      item.document.encryptedKey.assign(enckey.begin(), enckey.end());
      binarystring initvec((*it)["initVec"]);
      item.document.initialVector.assign(initvec.begin(), initvec.end());
      binarystring encdata((*it)["encData"]);
      item.document.encryptedData.assign(encdata.begin(), encdata.end());

      doclist->push_back(item);
    }
  }
  catch(PGSTD::runtime_error &ex)
  {
    LOG4CXX_ERROR(logger, "Unable to execute query: " << ex.what());
    throw runtime_error("Unable to execute query");
  }

  return doclist;
}

/*----------------------------------------------------------------------------*/

boost::shared_ptr<Ehr::DocumentList> DataManager::findDocuments(
    const Ehr::AccountIdentifier& owner, const Ehr::AccountIdentifier& requestor,
    const Ehr::UserType requestorType, bool hasFrom, Ehr::Timestamp from,
    bool hasTill, Ehr::Timestamp till, Ehr::DocumentType type, Ice::Long docID)
throw(runtime_error, Ehr::EhrException)
{
  LOG4CXX_TRACE(logger, "findDocuments(\"" << owner.user << "@" << owner.provider
      << "\", \"" << requestor.user << "@" << requestor.provider << "\", "
      << hasFrom << ", " << from.msecs << ", " << hasTill << ", " << till.msecs
      << ", docType, " << docID << ")");
  boost::shared_ptr<DocumentList> doclist(new DocumentList);

  try
  {
    ConnectionPool::Connection conn = getConnection();

    /* build dynamic WHERE-string */
    stringstream optstring("");
    if (hasFrom)
      optstring << "AND d.created >= to_timestamp(" << from.msecs << " / 1000) ";
    if (hasTill)
      optstring << "AND d.created <= to_timestamp(" << till.msecs << " / 1000) ";
    if (type == Ehr::DOCPRESCRIPTION || requestorType == PHARMACIST)
      optstring << "AND d.type = 'PRESCRIPTION' ";
    else if (type == Ehr::DOCPRESCRIPTION)
      optstring << "AND d.type = 'CONSUMEDPRESCRIPTION' ";
    if (docID > 0)
      optstring << "AND d.id = " << docID << " ";

    /*
     * get documentlist
     * this would be much prettier using prepare::invocation
     * but this crashes in pqxx <= 2.6.9
     */
    work T(*conn, "findDocuments");
    result res;
    conn->unprepare("findDocuments");
    if (owner.user == requestor.user && owner.provider == requestor.provider)
    {
      /* documentlist of own documents */
      conn->prepare("findDocuments",
          "SELECT *, FLOOR(EXTRACT(EPOCH FROM d.created) * 1000) AS msecs "
          "FROM documents d "
          "JOIN users u ON (d.userid = u.id AND u.name = $1) "
          "WHERE d.valid = TRUE " + optstring.str())
        ("TEXT", prepare::treat_string);
      res = T.prepared("findDocuments")
        (owner.user).exec();
    }
    else
    {
      /* documentlist of other person */
      conn->prepare("findDocuments",
          "SELECT *, FLOOR(EXTRACT(EPOCH FROM d.created) * 1000) AS msecs "
          "FROM documents d "
          "JOIN users u ON (d.userid = u.id AND u.name = $1) "
          "WHERE d.valid = TRUE "
          "AND (( "
          "  d.defacl = 'ALLOW' AND d.id NOT IN ( "
          "    SELECT p.docid "
          "    FROM permissions p "
          "    WHERE username = $2 AND provider = $3 AND acl = 'DENY') "
          ") OR ( "
          "  d.defacl = 'DENY' AND d.id IN ( "
          "    SELECT p.docid "
          "    FROM permissions p "
          "    WHERE username = $2 AND provider = $3 AND acl = 'ALLOW') "
          ")) " + optstring.str())
        ("TEXT", prepare::treat_string)
        ("TEXT", prepare::treat_string)
        ("TEXT", prepare::treat_string);
      res = T.prepared("findDocuments")
        (owner.user)
        (requestor.user)
        (requestor.provider).exec();
    }
    T.commit();

    /* add documents to doclist */
    for (result::const_iterator it = res.begin(); it != res.end(); ++it)
    {
      DocumentListItem item;
      item.id = (*it)["id"].as<Ice::Long>();
      item.created.msecs = (*it)["msecs"].as<Ice::Long>();
      binarystring enckey((*it)["encKey"]);
      item.document.encryptedKey.assign(enckey.begin(), enckey.end());
      binarystring initvec((*it)["initVec"]);
      item.document.initialVector.assign(initvec.begin(), initvec.end());
      binarystring encdata((*it)["encData"]);
      item.document.encryptedData.assign(encdata.begin(), encdata.end());

      doclist->push_back(item);
    }
  }
  catch(PGSTD::runtime_error &ex)
  {
    LOG4CXX_ERROR(logger, "Unable to execute query: " << ex.what());
    throw runtime_error("Unable to execute query");
  }

  return doclist;
}

/*----------------------------------------------------------------------------*/

void DataManager::setDefaultAccess(const Ehr::AccountIdentifier& requestor,
    Ice::Long docID, Ehr::AccessType accessType)
throw(runtime_error, Ehr::EhrException)
{
  LOG4CXX_TRACE(logger, "setDefaultAccess(\"" << requestor.user << "@" <<
      requestor.provider << "\", " << docID << ", accessType)");

  try
  {
    ConnectionPool::Connection conn = getConnection();

    work T(*conn, "setDefaultAccess");
    conn->prepare("setDefaultAccess",
        "UPDATE documents d "
        "SET defacl = $1 "
        "FROM users u "
        "WHERE d.id = $2 AND d.userid = u.id AND u.name = $3")
      ("VARCHAR", prepare::treat_string)
      ("INTEGER", prepare::treat_direct)
      ("TEXT",    prepare::treat_string);
    result res = T.prepared("setDefaultAccess")
      ((accessType == ALLOW) ? "ALLOW" : "DENY")
      (docID)
      (requestor.user).exec();
    T.commit();

    if (res.affected_rows() == 0)
      throw InvalidRequestException("Document ID is unknown");
  }
  catch(PGSTD::runtime_error &ex)
  {
    LOG4CXX_ERROR(logger, "Unable to execute query: " << ex.what());
    throw runtime_error("Unable to execute query");
  }
}

/*----------------------------------------------------------------------------*/

void DataManager::createPermission(const Ehr::AccountIdentifier& requestor,
    Ice::Long docID, const Ehr::AccountIdentifier& account, Ehr::AccessType accessType)
throw(runtime_error, Ehr::EhrException)
{
  LOG4CXX_TRACE(logger, "createPermission(\"" << requestor.user << "@"
      << requestor.provider << "\", " << docID << ", \"" << account.user << "@"
      << account.provider << "\", accessType)");

  try
  {
    ConnectionPool::Connection conn = getConnection();

    /* check document belonging to user */
    work T1(*conn, "getDocument");
    conn->prepare("getDocument",
        "SELECT d.id "
        "FROM documents d "
        "JOIN users u ON (d.userid = u.id AND u.name = $1) "
        "WHERE d.id = $2")
      ("VARCHAR", prepare::treat_string)
      ("INTEGER", prepare::treat_direct);
    result res = T1.prepared("getDocument")
      (requestor.user)
      (docID).exec();
    T1.commit();

    if (res.size() == 0)
      throw InvalidRequestException("Document ID is unknown");

    /* insert new permission */
    work T2(*conn, "createPermission");
    conn->prepare("createPermission",
        "INSERT INTO permissions "
        "VALUES (DEFAULT, $1, $2, $3, $4)")
      ("INTEGER", prepare::treat_direct)
      ("TEXT",    prepare::treat_string)
      ("TEXT",    prepare::treat_string)
      ("VARCHAR", prepare::treat_string);
    T2.prepared("createPermission")
      (docID)
      (account.user)
      (account.provider)
      ((accessType == ALLOW) ? "ALLOW" : "DENY").exec();
    T2.commit();
  }
  catch(PGSTD::runtime_error &ex)
  {
    LOG4CXX_ERROR(logger, "Unable to execute query: " << ex.what());
    throw runtime_error("Unable to execute query");
  }
}

/*----------------------------------------------------------------------------*/

boost::shared_ptr<Ehr::Permissions> DataManager::listPermissions(
    const Ehr::AccountIdentifier& requestor, Ice::Long docID)
throw(runtime_error, Ehr::EhrException)
{
  LOG4CXX_TRACE(logger, "listPermission(\"" << requestor.user << "@"
      << requestor.provider << "\", " << docID << ")");

  boost::shared_ptr<Permissions> permlist(new Permissions);

  try
  {
    ConnectionPool::Connection conn = getConnection();

    /* get default acl */
    work T1(*conn, "getDefaultAccess");
    conn->prepare("getDefaultAccess",
        "SELECT defacl "
        "FROM documents d "
        "JOIN users u ON (d.userid = u.id AND u.name = $1) "
        "WHERE d.id = $2")
      ("TEXT",    prepare::treat_string)
      ("INTEGER", prepare::treat_direct);
    result res = T1.prepared("getDefaultAccess")
      (requestor.user)
      (docID).exec();
    T1.commit();

    if (res.size() == 0)
      throw InvalidRequestException("Document ID is unknown");

    /* set default acl */
    permlist->defaultAccess = (res[0]["defacl"].as<string>() == "ALLOW") ? ALLOW : DENY;

    /*
     * get permissionlist of this document
     * docID is considered safe due to the join in the query above
     */
    work T2(*conn, "listPermissions");
    conn->prepare("listPermissions",
        "SELECT * "
        "FROM permissions p "
        "WHERE p.docid = $1")
      ("INTEGER", prepare::treat_direct);
    res = T2.prepared("listPermissions")
      (docID).exec();
    T2.commit();

    /* add permissions to permlist */
    for (result::const_iterator it = res.begin(); it != res.end(); ++it)
    {
      AccessControlListItem item;
      item.id               = (*it)["id"].as<Ice::Long>();
      item.account.user     = (*it)["username"].as<string>();
      item.account.provider = (*it)["provider"].as<string>();
      item.access           = ((*it)["acl"].as<string>() == "ALLOW") ? ALLOW : DENY;
      permlist->acl.push_back(item);
    }
  }
  catch(PGSTD::runtime_error &ex)
  {
    LOG4CXX_ERROR(logger, "Unable to execute query: " << ex.what());
    throw runtime_error("Unable to execute query");
  }

  return permlist;
}

/*----------------------------------------------------------------------------*/

void DataManager::removePermission(const Ehr::AccountIdentifier& requestor,
    Ice::Long docID, Ice::Long permissionID)
throw(runtime_error, Ehr::EhrException)
{
  LOG4CXX_TRACE(logger, "removePermission(\"" << requestor.user << "@"
      << requestor.provider << "\", " << docID << ", " << permissionID << ")");

  try
  {
    ConnectionPool::Connection conn = getConnection();

    work T(*conn, "removePermission");
    conn->prepare("removePermission",
        "DELETE FROM permissions p "
        "USING documents d, users u "
        "WHERE p.id = $1 AND p.docid = $2 "
        "  AND p.docid = d.id AND d.userid = u.id AND u.name = $3")
      ("INTEGER", prepare::treat_direct)
      ("INTEGER", prepare::treat_direct)
      ("TEXT",    prepare::treat_string);
    result res = T.prepared("removePermission")
      (permissionID)
      (docID)
      (requestor.user).exec();
    T.commit();

    if (res.affected_rows() == 0)
      throw InvalidRequestException("Permission ID is unknown");
  }
  catch(PGSTD::runtime_error &ex)
  {
    LOG4CXX_ERROR(logger, "Unable to execute query: " << ex.what());
    throw runtime_error("Unable to execute query");
  }
}

/*----------------------------------------------------------------------------*/

const Ehr::UserType DataManager::getUserType(const std::string& name)
  throw(runtime_error, Ehr::EhrException)
{
  LOG4CXX_TRACE(logger, "getUserType(\"" << name << "\")");
  UserType type = PATIENT;

  try
  {
    ConnectionPool::Connection conn = getConnection();

    work T(*conn, "getUsertype");
    conn->prepare("getUsertype",
        "SELECT type "
        "FROM users "
        "WHERE name = $1")
      ("TEXT", prepare::treat_string);
    result res = T.prepared("getUsertype")
      (name).exec();
    T.commit();

    if (res.size() == 0)
      throw InvalidRequestException("Unknown user");

    string stype = res[0]["type"].as<string>();
    type = (stype == "PHYSICAN") ? PHYSICAN : ((stype == "PHARMACIST") ? PHARMACIST : PATIENT);
  }
  catch(PGSTD::runtime_error &ex)
  {
    LOG4CXX_ERROR(logger, "Unable to execute query: " << ex.what());
    throw runtime_error("Unable to execute query");
  }

  return type;
}

/*----------------------------------------------------------------------------*/

void DataManager::createUser(const std::string& name, const Ehr::UserType& type)
throw(runtime_error)
{
  LOG4CXX_TRACE(logger, "createUser(\"" << name << "\", ...)");

  try
  {
    ConnectionPool::Connection conn = getConnection();

    work T(*conn, "createUser");
    conn->prepare("createUser",
        "INSERT INTO users "
        "VALUES (DEFAULT, $1, $2)")
      ("TEXT",     prepare::treat_string)
      ("VARCHAR",  prepare::treat_string);
    T.prepared("createUser")
      (name)
      ((type == PHYSICAN) ? "PHYSICAN" :
       ((type == PHARMACIST) ? "PHARMACIST" : "PATIENT")).exec();
    T.commit();
  }
  catch(PGSTD::runtime_error &ex)
  {
    LOG4CXX_ERROR(logger, "Unable to execute query: " << ex.what());
    throw runtime_error("Unable to execute query");
  }
}

/*----------------------------------------------------------------------------*/

void DataManager::emptyTables()
throw(runtime_error)
{
  LOG4CXX_TRACE(logger, "emptyTables()");

  try
  {
    ConnectionPool::Connection conn = getConnection();

    work T(*conn, "emptyTables");
    T.exec("DELETE FROM permissions");
    T.exec("DELETE FROM documents");
    T.exec("DELETE FROM users");
    T.commit();
  }
  catch(PGSTD::runtime_error &ex)
  {
    LOG4CXX_ERROR(logger, "Unable to execute query: " << ex.what());
    throw runtime_error("Unable to execute query");
  }
}

/*----------------------------------------------------------------------------*/

ConnectionPool::Connection DataManager::getConnection()
throw(runtime_error)
{
  if (m_pool == NULL)
  {
    LOG4CXX_ERROR(logger, "Got ehr request but no connection to database");
    throw runtime_error("No connection to database");
  }
  return m_pool->get();
}

/*----------------------------------------------------------------------------*/

const std::vector<Ice::Byte> DataManager::convertToByteStream(const Ice::ObjectPtr& request)
{
  /* in order to sign our request, we need to convert
   * the request to a bytestream first
   */
  Ice::OutputStreamPtr out = Ice::createOutputStream(m_comm);
  vector<Ice::Byte> data;
  out->writeObject(request);
  out->writePendingObjects();
  out->finished(data);
  return data;
}

/* vim: set et sw=2 ts=2: */
