/**
 * $Id: datamanager.h 100 2010-01-18 03:20:47Z l0728348 $
 *
 * Copyright 2009
 *
 * @author Christian, Bernhard, Manuel / Gruppe 1
 * @brief Interface for accessing data
 */

#ifndef PROVIDERDATA_H
#define PROVIDERDATA_H

#include "configuration.h"
#include "connectionpool.h"
#include <ehr.h>
#include <stdexcept>
#include <string>
#include <vector>
#include <boost/shared_ptr.hpp>
#include <Ice/Ice.h>
#include <IceUtil/Mutex.h>
#include <IceUtil/Monitor.h>


/**
 * Interface for accessing misc data such as
 * database and files
 *
 * This interface just gets the data, or processes it,
 * it trusts the input. It is only checked if the data is
 * valid when necessary.
 */
class DataManager
{
  public:
    /**
     * Exception thrown by DataManager
     */
    class runtime_error
    : public std::runtime_error
    {
      public:
        /**
         * Default exception ctor
         * @param  what message to pass along
         */
        runtime_error(const std::string& what)
          : std::runtime_error(what)
        {}
    };

    /**
     * default ctor
     */
    DataManager(Configuration *conf, Ice::CommunicatorPtr comm = NULL);

    /**
     * default dtor
     */
    ~DataManager()
    {
      disconnect();
    }

    /**
     * connects to backend database
     * @return true on success, false otherwise
     */
    bool connect();

    /**
     * disconnect from backend database
     * @return true on success, false otherwise
     */
    bool disconnect();

    /**
     * get error message from last error
     * @return error message from last error
     */
    const std::string getLastError()
    {
      return m_lasterror.str();
    }

    /**
     * check if an error occured during parsing
     * @return true on error, false otherwise
     */
    const bool operator!()
    {
      return !m_lasterror.str().empty();
    }

    /**
     * sets pointer to ice communicator for internal use
     * this is needed for creating streams out of requests
     * @param  comm pointer to ice communicator
     */
    void setCommunicator(Ice::CommunicatorPtr comm)
    {
      m_comm = comm;
    }

    /**
     * get pointer to current configuration object
     * @return pointer to configuration object
     */
    Configuration *getConfiguration()
    {
      return m_conf;
    }

    /**
     * Checks the signature of a user for a request
     * Works only for local users.
     * @param  user username (part of AccountIdentifier)
     * @param  signature signature to check
     * @param  req data which should be verified
     * @return true if signature is ok
     * @throw  Ehr::InvalidAccountException on unknown user
     */
    bool checkSignature(const std::string& user, const Ehr::Signature& signature,
        Ehr::RequestPtr req)
      throw(Ehr::EhrException);

    /**
     * Checks the signature of a provider for a request
     * @param  provider providername (part of AccountIdentifier)
     * @param  signature signature to check
     * @param  req data which should be verified
     * @return true if signature is ok
     * @throw  Ehr::InvalidAccountException on unknown user
     */
    bool checkProviderSignature(const std::string& provider,
        const Ehr::Signature& signature, Ehr::RequestPtr req)
      throw(Ehr::EhrException);

    /**
     * Create signature of request by providers key
     * @param  req data which should signed
     * @param  signature output paramater for signature
     * @throw  runtime_error on internal error
     */
    void signProviderRequest(const Ehr::RequestPtr& req,
        Ehr::Signature& signature)
      throw(runtime_error);

    /**
     * Creates a prescription
     * @param  doc encrypted document to store
     * @param  owner account that owns the document
     * @throw  runtime_error on internal error
     */
    void createPrescription(const Ehr::EncryptedDocument& doc,
        const Ehr::AccountIdentifier& owner)
      throw(runtime_error, Ehr::EhrException);

    /**
     * Consumes a prescription
     * @param  doc consumed prescription document
     * @param  prescriptionToConsume ID of prescription that is consumed
     * @param  owner account that owns the document
     * @param  requestor requestor of the documents
     * @throw  Ehr::PermissionDeniedException if access by requestor is not allowed
     * @throw  runtime_error on internal error
     */
    void consumePrescription(const Ehr::EncryptedDocument& doc,
        Ice::Long prescriptionToConsume, const Ehr::AccountIdentifier& owner,
        const Ehr::AccountIdentifier& requestor)
      throw(runtime_error, Ehr::EhrException);

    /**
     * Lists all documents of an owner, that the requestor may see
     * @param  owner owner of the documents
     * @param  requestor requestor of the documents
     * @param  requestorType usertype of requestor
     * @return list of documents as smart pointer
     * @throw  runtime_error on internal error
     */
    boost::shared_ptr<Ehr::DocumentList> listDocuments(
        const Ehr::AccountIdentifier& owner,
        const Ehr::AccountIdentifier& requestor,
        const Ehr::UserType requestorType)
      throw(runtime_error, Ehr::EhrException);

    /**
     * Finds all documents that match certain criteria
     * @param  owner owner of the documents
     * @param  requestor requestor of the documents
     * @param  requestorType usertype of requestor
     * @param  hasFrom whether from param  is valid
     * @param  from only include documents older
     * @param  hasTill whether till param  is valid
     * @param  till only include documents younger
     * @param  type type of documents to find
     * @param  docID search for this document ID (0 means any document)
     * @throw  runtime_error on internal error
     */
    boost::shared_ptr<Ehr::DocumentList> findDocuments(
        const Ehr::AccountIdentifier& owner,
        const Ehr::AccountIdentifier& requestor,
        const Ehr::UserType requestorType, bool hasFrom, Ehr::Timestamp from,
        bool hasTill, Ehr::Timestamp till, Ehr::DocumentType type,
        Ice::Long docID)
      throw(runtime_error, Ehr::EhrException);

    /**
     * Sets the default access type for a document
     * @param  requestor requestor of the action
     * @param  docID document to set access for
     * @param  accessType new access type for document
     * @throw  Ehr::InvalidRequestException if docID is not valid
     * @throw  runtime_error on internal error
     */
    void setDefaultAccess(const Ehr::AccountIdentifier& requestor,
        Ice::Long docID, Ehr::AccessType accessType)
      throw(runtime_error, Ehr::EhrException);

    /**
     * Sets a permission for a document for a user
     * @param  requestor requestor of the action
     * @param  docID document to set access for
     * @param  account user to set access for
     * @param  accessType access type for permission
     * @throw  Ehr::InvalidRequestException if docID is not valid
     * @throw  runtime_error on internal error
     */
    void createPermission(const Ehr::AccountIdentifier& requestor, Ice::Long docID,
        const Ehr::AccountIdentifier& account, Ehr::AccessType accessType)
      throw(runtime_error, Ehr::EhrException);

    /**
     * Lists permissions for a document
     * @param  requestor requestor of the action
     * @param  docID document to set access for
     * @throw  Ehr::InvalidRequestException if docID is not valid
     * @throw  runtime_error on internal error
     */
    boost::shared_ptr<Ehr::Permissions> listPermissions(
        const Ehr::AccountIdentifier& requestor, Ice::Long docID)
      throw(runtime_error, Ehr::EhrException);

    /**
     * Removes a permission for a document
     * @param  requestor requestor of the action
     * @param  docID document to set access for
     * @param  permissionID id of the permission to remove
     * @throw  Ehr::InvalidRequestException if docID is not valid
     * @throw  runtime_error on internal error
     */
    void removePermission(const Ehr::AccountIdentifier& requestor,
        Ice::Long docID, Ice::Long permissionID)
      throw(runtime_error, Ehr::EhrException);

    /**
     * Get type (or role) of user
     * @param  name name of user
     * @return type (or role) of user
     * @throw Ehr::InvalidRequestException if name is not valid
     * @throw runtime_error on internal error
     */
    const Ehr::UserType getUserType(const std::string& name)
      throw(runtime_error, Ehr::EhrException);

    /**
     * create new user for current provider
     * make sure that certificates are available too!
     * @param  name name of new user
     * @param  type type (or role) of new user
     * @throw runtime_error on internal error
     */
    void createUser(const std::string& name, const Ehr::UserType& type)
      throw(runtime_error);

    /**
     * empty all used tables. this is for testing purpose only
     * @throw runtime_error on internal error
     */
    void emptyTables()
      throw(runtime_error);

  protected:
    /**
     * get available connection from connection pool
     * @return connection to database
     * @throw runtime_error if no connection is available
     */
    ConnectionPool::Connection getConnection()
      throw(runtime_error);

    /**
     * convert ice request to bytestream
     * @param  request ice request to convert
     * @return converted bytestream containing the request
     */
    const std::vector<Ice::Byte> convertToByteStream(const Ice::ObjectPtr& request);

  private:
    std::stringstream m_lasterror;
    Configuration *m_conf;
    ConnectionPool *m_pool;
    Ice::CommunicatorPtr m_comm;
    IceUtil::Mutex m_mutex;
};

#endif

/* vim: set et sw=2 ts=2: */
