/**
 * $Id: ehrprovider.cpp 145 2010-01-21 00:34:26Z l0728348 $
 *
 * Copyright 2009
 *
 * @author Christian, Bernhard, Manuel / Gruppe 1
 * @brief Main class
 */

#include "logger.h"
#include <log4cxx/propertyconfigurator.h>
#include <log4cxx/consoleappender.h>
#include <log4cxx/patternlayout.h>
#include <log4cxx/helpers/exception.h>
#include <iostream>
#include <unistd.h>
#include <string.h>
#include <Ice/Plugin.h>
#include "ehrprovider.h"
#include "configuration.h"
#include "datamanager.h"
#include "provideri.h"
#include "interprovideri.h"

using namespace std;
using namespace Ehr;

namespace
{
  log4cxx::LoggerPtr logger(log4cxx::Logger::getLogger("EhrProvider"));
}

/*----------------------------------------------------------------------------*/

EhrProvider::~EhrProvider()
{
  if (!log4cxx::Logger::getRootLogger()->getAllAppenders().empty())
    LOG4CXX_TRACE(logger, "~EhrProvider()");
  cleanup();
}

/*----------------------------------------------------------------------------*/

int EhrProvider::setup(int argc, char *argv[])
{
  if (!log4cxx::Logger::getRootLogger()->getAllAppenders().empty())
    LOG4CXX_TRACE(logger, "setup()");
  cleanup();

  /* parse commandline optioons, load configuration */
  m_conf = new Configuration(argc, argv);
  if (m_conf->shouldExit())
    return 0;
  if (!*m_conf)
  {
    cerr << "Error: " << m_conf->getLastError() << endl;
    return 1;
  }

  /* initialize logger */
  try
  {
    log4cxx::PropertyConfigurator::configure(m_conf->getConfFile());
  }
  catch(log4cxx::helpers::Exception &ex)
  {
#if __LOG4CXX_VERSION >= 1000
    cerr << "Error: " << ex.what() << endl;
#else
    cerr << "Error: " << ex.getMessage() << endl;
#endif
    return 1;
  }

  /* send logs to stderr upon request */
  if (m_conf->hasVal("e"))
  {
    /* remove all appenders */
    log4cxx::LoggerPtr root = log4cxx::Logger::getRootLogger();
    root->removeAllAppenders();

    /* add our appender */
    log4cxx::ConsoleAppender *appender = new log4cxx::ConsoleAppender(
        log4cxx::LayoutPtr(new log4cxx::PatternLayout("%-4r [%t] %-5p %c %x - %m%n")), "System.err");
#if __LOG4CXX_VERSION >= 1000
    log4cxx::helpers::Pool p;
    appender->activateOptions(p);
#else
    appender->activateOptions();
#endif
    root->addAppender(appender);
  }

  /* enable trace mod */
  if (m_conf->hasVal("t"))
  {
#if __LOG4CXX_VERSION >= 1000
    log4cxx::Logger::getRootLogger()->setLevel(log4cxx::Level::getTrace());
#else
    log4cxx::Logger::getRootLogger()->setLevel(log4cxx::CustomLevel::TRACE);
#endif
  }
  /* enable debug mode */
  else if (m_conf->hasVal("d"))
  {
#if __LOG4CXX_VERSION >= 1000
    log4cxx::Logger::getRootLogger()->setLevel(log4cxx::Level::getDebug());
#else
    log4cxx::Logger::getRootLogger()->setLevel(log4cxx::Level::DEBUG);
#endif
  }

  /* set ice logger */
  Ice::setProcessLogger(new IceLogger());

  /* print debug messages - AFTER logger has been initialized!! */
  LOG4CXX_DEBUG(logger, "Configuration file \"" << m_conf->getConfFile() << "\" loaded");
  LOG4CXX_DEBUG(logger, "Logger initialised sucessfully");

  /* load certificates */
  if (!m_conf->loadCertificates())
    return 1;

  /* setup datamanager */
  m_data = new DataManager(m_conf);
  if (!*m_data || !m_data->connect())
  {
    cerr << "Error: " << m_data->getLastError() << endl;
    return 1;
  }

  /* create our objects */
  m_provider = new ProviderI(m_data);
  m_interprovider = new InterProviderI(m_data);

  /* detach if requested - AFTER everything is initialized */
  if (m_conf->hasVal("D"))
  {
    if (daemon(1, 1) == -1)
    {
      LOG4CXX_FATAL(logger, "Error: Unable to detach from foreground: " << strerror(errno));
      return 1;
    }
  }

  return -1;
}

/*----------------------------------------------------------------------------*/

void EhrProvider::cleanup()
{
  if (!log4cxx::Logger::getRootLogger()->getAllAppenders().empty())
    LOG4CXX_TRACE(logger, "cleanup()");

  if (m_data)
    delete m_data;
  m_data = NULL;

  if (m_conf)
    delete m_conf;
  m_conf = NULL;
}

/*----------------------------------------------------------------------------*/

int EhrProvider::run(int argc, char *argv[])
{
  if (!log4cxx::Logger::getRootLogger()->getAllAppenders().empty())
    LOG4CXX_TRACE(logger, "run()");

  LOG4CXX_INFO(logger, "Initializing EhrProvider..");

  /* set certificates */
  Ice::PropertiesPtr props = communicator()->getProperties();
  props->setProperty("IceSSL.CertAuthFile",
      m_conf->getConfPath(m_conf->val<string>("certificates.rootca")));
  props->setProperty("IceSSL.CertFile",
      m_conf->getConfPath(m_conf->val<string>("certificates.mycert")));
  props->setProperty("IceSSL.KeyFile",
      m_conf->getConfPath(m_conf->val<string>("certificates.mykey")));

  /* set communicator */
  m_data->setCommunicator(communicator());

  /* load plugins */
  communicator()->getPluginManager()->initializePlugins();

  /* setup endpoint */
  string endpoint = m_conf->val<string>("network.endpoint");
  LOG4CXX_DEBUG(logger, "Setting up Provider Adapter using endpoint \"" << endpoint << "\"");
  Ice::ObjectAdapterPtr providerAdapter = communicator()->createObjectAdapterWithEndpoints(
      "ProviderAdapter", endpoint);
  providerAdapter->add(m_provider, communicator()->stringToIdentity("Provider"));
  providerAdapter->add(m_interprovider, communicator()->stringToIdentity("InterProvider"));
  providerAdapter->activate();

  LOG4CXX_INFO(logger, "EhrProvider initialisation completed");
  communicator()->waitForShutdown();

  if (interrupted())
    LOG4CXX_WARN(logger, "Interrupted, shutting down gracefully");
  return 0;
}

/* vim: set et sw=2 ts=2: */
