/**
 * $Id: interprovideri.cpp 145 2010-01-21 00:34:26Z l0728348 $
 *
 * Copyright 2009
 *
 * @author Christian, Bernhard, Manuel / Gruppe 1
 * @brief Implementation of Client to Provider Interface
 */

#include "interprovideri.h"
#include "util.h"
#include "logger.h"

namespace
{
  log4cxx::LoggerPtr logger(log4cxx::Logger::getLogger("InterProviderI"));
}

/*----------------------------------------------------------------------------*/

void InterProviderI::createPrescription(
    const ::Ehr::CreatePrescriptionRequestPtr& request,
    const ::Ehr::Signature& providerSignature,
    const ::Ehr::Signature& ownerSignatur,
    const Ice::Current& current)
throw(Ehr::EhrException)
{
  LOG4CXX_TRACE(logger, "createPrescription()");

  checkRequest(request, providerSignature, ownerSignatur);
  try
  {
    m_data->createPrescription(request->prescription, request->owner);
  }
  catch (const DataManager::runtime_error& e)
  {
    throw Ehr::EhrException("Internal error");
  }
}

/*----------------------------------------------------------------------------*/

void InterProviderI::consumePrescription(
    const ::Ehr::ConsumePrescriptionRequestPtr& request,
    const ::Ehr::Signature& providerSignature,
    const ::Ehr::Signature& ownerSignatur,
    const Ice::Current& current)
throw(Ehr::EhrException)
{
  LOG4CXX_TRACE(logger, "consumePrescription()");

  checkRequest(request, providerSignature, ownerSignatur);
  try
  {
    m_data->consumePrescription(request->consumedPrescription,
        request->prescriptionId, request->owner, request->requestor);
  }
  catch (const DataManager::runtime_error& e)
  {
    throw Ehr::EhrException("Internal error");
  }
}

/*----------------------------------------------------------------------------*/

::Ehr::DocumentList InterProviderI::listDocuments(
    const ::Ehr::ListDocumentsRequestPtr& request,
    const ::Ehr::Signature& providerSignature,
    const ::Ehr::Signature& ownerSignatur,
    ::Ehr::UserType requestorUsertype,
    const Ice::Current& current)
throw(Ehr::EhrException)
{
  LOG4CXX_TRACE(logger, "listDocuments()");

  checkRequest(request, providerSignature, ownerSignatur);
  try
  {
    boost::shared_ptr<Ehr::DocumentList> p = m_data->listDocuments(
        request->owner, request->requestor, requestorUsertype);
    return *p;
  }
  catch (const DataManager::runtime_error& e)
  {
    throw Ehr::EhrException("Internal error");
  }
}

/*----------------------------------------------------------------------------*/

::Ehr::DocumentList InterProviderI::findDocuments(
    const ::Ehr::FindDocumentsRequestPtr& request,
    const ::Ehr::Signature& providerSignature,
    const ::Ehr::Signature& ownerSignatur,
    ::Ehr::UserType requestorUsertype,
    const Ice::Current& current)
throw(Ehr::EhrException)
{
  LOG4CXX_TRACE(logger, "findDocuments()");

  checkRequest(request, providerSignature, ownerSignatur);
  try
  {
    boost::shared_ptr<Ehr::DocumentList> p = m_data->findDocuments(
        request->owner, request->requestor, requestorUsertype,
        request->hasFrom, request->from, request->hasTill, request->till,
        request->type, request->documentId);
    return *p;
  }
  catch (const DataManager::runtime_error& e)
  {
    throw Ehr::EhrException("Internal error");
  }
}

/*----------------------------------------------------------------------------*/

void InterProviderI::checkRequest(Ehr::ThirdPartyRequestPtr req,
    const Ehr::Signature& providerSignature,
    const Ehr::Signature& ownerSignature)
throw(Ehr::EhrException)
{
  LOG4CXX_TRACE(logger, "checkRequest()");

  try
  {
    /* check provider signature */
    bool providerSigValid = m_data->checkProviderSignature(
        req->requestor.provider,providerSignature, req);
    if (!providerSigValid)
    {
      LOG4CXX_DEBUG(logger, "Got invalid provider signature");
      throw Ehr::SignatureException("Invalid provider signature");
    }

    /* check owner signature */
    bool ownerSigValid = m_data->checkSignature(req->owner.user,
        ownerSignature, req);
    if (!ownerSigValid)
    {
      LOG4CXX_DEBUG(logger, "Got invalid owner signature");
      throw Ehr::SignatureException("Invalid owner signature");
    }
  }
  catch (const DataManager::runtime_error& e)
  {
    throw Ehr::EhrException("Internal error");
  }

  /* check timestamp */
  if (req->when.msecs < (Util::getTime() - 60*1000))
  {
    LOG4CXX_DEBUG(logger, "Request with outdated timestamp");
    throw Ehr::InvalidRequestException("Invalid timestamp");
  }

  //TODO: check timestamp reuse
}

/* vim: set et sw=2 ts=2: */
