/**
 * $Id: provideri.cpp 173 2010-01-30 18:18:56Z l0707663 $
 *
 * Copyright 2009
 *
 * @author Christian, Bernhard, Manuel
 * @brief Implementation of Client to Provider Interface
 */

#include "provideri.h"
#include "util.h"
#include "logger.h"
#include <boost/shared_ptr.hpp>

namespace
{
  log4cxx::LoggerPtr logger(log4cxx::Logger::getLogger("ProviderI"));
}

/*----------------------------------------------------------------------------*/

void ProviderI::createPrescription(
    const ::Ehr::CreatePrescriptionRequestPtr& request,
    const ::Ehr::Signature& requestorSignature,
    const ::Ehr::Signature& ownerSignature,
    const Ice::Current& current)
throw(internal_error, Ehr::EhrException)
{
  LOG4CXX_TRACE(logger, "createPrescription()");

  /* validate requestor (must be possible locally) */
  checkRequest(request, requestorSignature);
  try
  {
    Ehr::UserType requsertype = m_data->getUserType(request->requestor.user);

    /*
     * the requestor is our member, check it's usertype
     * only physicans are allowed to create prescriptions
     */
    if (request->requestor.provider == m_data->getConfiguration()->getMyName() &&
        requsertype != Ehr::PHYSICAN)
      throw Ehr::PermissionDeniedException("You are not allowed to create prescriptions");

    if (request->owner.provider == m_data->getConfiguration()->getMyName())
    {
      /* it's a local request */
      LOG4CXX_DEBUG(logger, "LocalRequest: createPrescription()");

      checkRequestOwner(request, ownerSignature);
      m_data->createPrescription(request->prescription, request->owner);
    }
    else
    {
      /* it's a interprovider request */
      LOG4CXX_DEBUG(logger, "InterproviderRequest: createPrescription()");

      Ehr::InterProviderPrx interprovider;
      createInterProviderRequest(request->owner.provider,
          current.adapter->getCommunicator(), interprovider);
      Ehr::Signature signature;
      m_data->signProviderRequest(request, signature);
      interprovider->createPrescription(request, signature, ownerSignature);
    }
  }
  catch(const DataManager::runtime_error& e)
  {
    throw Ehr::EhrException("Internal error");
  }
}

/*----------------------------------------------------------------------------*/

void ProviderI::consumePrescription(
    const ::Ehr::ConsumePrescriptionRequestPtr& request,
    const ::Ehr::Signature& requestorSignature,
    const ::Ehr::Signature& ownerSignature,
    const Ice::Current& current)
throw(internal_error, Ehr::EhrException)
{
  LOG4CXX_TRACE(logger, "consumePrescription()");

  checkRequest(request, requestorSignature);
  try
  {
    Ehr::UserType requsertype = m_data->getUserType(request->requestor.user);

    /*
     * the requestor is our member, check it's usertype
     * only physicans and pharmacists are allowed to consume prescriptions
     */
    if (request->requestor.provider == m_data->getConfiguration()->getMyName() &&
        requsertype == Ehr::PATIENT)
      throw Ehr::PermissionDeniedException("You are not allowed to consume prescriptions");

    if (request->owner.provider == m_data->getConfiguration()->getMyName())
    {
      /* it's a local request */
      LOG4CXX_DEBUG(logger, "LocalRequest: consumePrescription()");

      checkRequestOwner(request, ownerSignature);
      m_data->consumePrescription(request->consumedPrescription,
          request->prescriptionId, request->owner, request->requestor);
    }
    else
    {
      /* it's a interprovider request */
      LOG4CXX_DEBUG(logger, "InterproviderRequest: consumePrescription()");

      Ehr::InterProviderPrx interprovider;
      createInterProviderRequest(request->owner.provider,
          current.adapter->getCommunicator(), interprovider);
      Ehr::Signature signature;
      m_data->signProviderRequest(request, signature);
      interprovider->consumePrescription(request, signature, ownerSignature);
    }
  }
  catch(const DataManager::runtime_error& e)
  {
    throw Ehr::EhrException("Internal error");
  }
}

/*----------------------------------------------------------------------------*/

::Ehr::DocumentList ProviderI::listDocuments(
    const ::Ehr::ListDocumentsRequestPtr& request,
    const ::Ehr::Signature& requestorSignature,
    const ::Ehr::Signature& ownerSignature,
    const Ice::Current& current)
throw(internal_error, Ehr::EhrException)
{
  LOG4CXX_TRACE(logger, "listDocuments()");

  checkRequest(request, requestorSignature);
  try
  {
    Ehr::UserType requsertype = m_data->getUserType(request->requestor.user);

    /*
     * the requestor is our member, check it's usertype
     * only physicans and pharmacists are allowed to view others documents
     * users are allowed to view their own documents
     */
    if (request->requestor.provider == m_data->getConfiguration()->getMyName() &&
        request->owner.user != request->requestor.user &&
        requsertype == Ehr::PATIENT)
      throw Ehr::PermissionDeniedException("You are not allowed to view others documents");

    if (request->owner.provider == m_data->getConfiguration()->getMyName())
    {
      /* it's a local request */
      LOG4CXX_DEBUG(logger, "LocalRequest: listDocuments()");

      checkRequestOwner(request, ownerSignature);
      /* get a shared ptr from the data layer */
      boost::shared_ptr<Ehr::DocumentList> l = m_data->listDocuments(
          request->owner, request->requestor, requsertype);
      /* copy it for ice, as required by method signature */
      return *l;
    }
    else
    {
      /* it's a interprovider request */
      LOG4CXX_DEBUG(logger, "InterproviderRequest: listDocuments()");

      Ehr::InterProviderPrx interprovider;
      createInterProviderRequest(request->owner.provider,
          current.adapter->getCommunicator(), interprovider);
      Ehr::Signature signature;
      m_data->signProviderRequest(request, signature);
      return interprovider->listDocuments(request, signature, ownerSignature,
          requsertype);
    }
  }
  catch(const DataManager::runtime_error& e)
  {
    throw Ehr::EhrException("Internal error");
  }
}

/*----------------------------------------------------------------------------*/

::Ehr::DocumentList ProviderI::findDocuments(
    const ::Ehr::FindDocumentsRequestPtr& request,
    const ::Ehr::Signature& requestorSignature,
    const ::Ehr::Signature& ownerSignature,
    const Ice::Current& current)
throw(internal_error, Ehr::EhrException)
{
  LOG4CXX_TRACE(logger, "findDocuments()");

  checkRequest(request, requestorSignature);
  try
  {
    Ehr::UserType requsertype = m_data->getUserType(request->requestor.user);

    /*
     * the requestor is our member, check it's usertype
     * only physicans and pharmacists are allowed to view others documents
     * users are allowed to view their own documents
     */
    if (request->requestor.provider == m_data->getConfiguration()->getMyName() &&
        request->owner.user != request->requestor.user &&
        requsertype == Ehr::PATIENT)
      throw Ehr::PermissionDeniedException("You are not allowed to view others documents");

    if (request->owner.provider == m_data->getConfiguration()->getMyName())
    {
      /* it's a local request */
      LOG4CXX_DEBUG(logger, "LocalRequest: findDocuments()");

      checkRequestOwner(request, ownerSignature);
      /* get a shared ptr from the data layer */
      boost::shared_ptr<Ehr::DocumentList> l = m_data->findDocuments(
          request->owner,request->requestor, requsertype,
          request->hasFrom, request->from, request->hasTill, request->till,
          request->type, request->documentId);
      /* copy it for ice, as required by method signature */
      return *l;
    }
    else
    {
      /* it's a interprovider request */
      LOG4CXX_DEBUG(logger, "InterproviderRequest: findDocuments()");

      Ehr::InterProviderPrx interprovider;
      createInterProviderRequest(request->owner.provider,
          current.adapter->getCommunicator(), interprovider);
      Ehr::Signature signature;
      m_data->signProviderRequest(request, signature);
      return interprovider->findDocuments(request, signature, ownerSignature,
          requsertype);
    }
  }
  catch(const DataManager::runtime_error& e)
  {
    throw Ehr::EhrException("Internal error");
  }
}

/*----------------------------------------------------------------------------*/

void ProviderI::setDefaultAccess(
    const ::Ehr::SetDefaultAccessRequestPtr& request,
    const ::Ehr::Signature& requestorSignature,
    const Ice::Current& current)
throw(internal_error, Ehr::EhrException)
{
  LOG4CXX_TRACE(logger, "setDefaultAccess()");

  checkRequest(request, requestorSignature);
  try
  {
    m_data->setDefaultAccess(request->requestor, request->documentId, request->access);
  }
  catch (const DataManager::runtime_error& e)
  {
    throw Ehr::EhrException("Internal error");
  }
}

/*----------------------------------------------------------------------------*/

void ProviderI::createPermission(
    const ::Ehr::CreatePermissionRequestPtr& request,
    const ::Ehr::Signature& requestorSignature,
    const Ice::Current& current)
throw(internal_error, Ehr::EhrException)
{
  LOG4CXX_DEBUG(logger, "createPermission");

  checkRequest(request, requestorSignature);
  try
  {
    m_data->createPermission(request->requestor, request->documentId,
        request->account, request->access);
  }
  catch (const DataManager::runtime_error& e)
  {
    throw Ehr::EhrException("Internal error");
  }
}

/*----------------------------------------------------------------------------*/

::Ehr::Permissions ProviderI::listPermissions(
    const ::Ehr::ListPermissionsRequestPtr& request,
    const ::Ehr::Signature& requestorSignature,
    const Ice::Current& current)
throw(internal_error, Ehr::EhrException)
{
  LOG4CXX_TRACE(logger, "listPermissions()");

  checkRequest(request, requestorSignature);
  try
  {
    /* get a shared ptr from the data layer */
    boost::shared_ptr<Ehr::Permissions> p = m_data->listPermissions(
        request->requestor, request->documentId);
    /* copy for ice as required by method signature */
    return *p;
  }
  catch (const DataManager::runtime_error& e)
  {
    throw Ehr::EhrException("Internal error");
  }
}

/*----------------------------------------------------------------------------*/

void ProviderI::removePermission(
    const ::Ehr::RemovePermissionRequestPtr& request,
    const ::Ehr::Signature& requestorSignature,
    const Ice::Current& current)
throw(internal_error, Ehr::EhrException)
{
  LOG4CXX_TRACE(logger, "removePermission()");

  checkRequest(request, requestorSignature);
  try
  {
    m_data->removePermission(request->requestor, request->documentId,
        request->permissionId);
  }
  catch (const DataManager::runtime_error& e)
  {
    throw Ehr::EhrException("Internal error");
  }
}

/*----------------------------------------------------------------------------*/

void ProviderI::checkRequest(Ehr::RequestPtr request,
    const Ehr::Signature& requestorSig)
throw(Ehr::EhrException)
{
  LOG4CXX_TRACE(logger, "checkRequest()");

  /* check requestor */
  bool sigvalid = m_data->checkSignature(request->requestor.user, requestorSig, request);
  if (!sigvalid)
    throw Ehr::SignatureException("Invalid requestor");

  /* check timestamp */
  if (request->when.msecs < ( Util::getTime() - 60*1000))
  {
    LOG4CXX_ERROR(logger, "Request with outdated timestamp");
    throw Ehr::InvalidRequestException("Invalid timestamp");
  }

  //TODO: (timestamp_last_request_from_requestor < when)
}

/*----------------------------------------------------------------------------*/

void ProviderI::checkRequestOwner(Ehr::ThirdPartyRequestPtr request,
    const Ehr::Signature& ownerSig)
throw(Ehr::EhrException)
{
  LOG4CXX_TRACE(logger, "checkRequestOwner()");

  bool sigvalid = m_data->checkSignature(request->owner.user, ownerSig, request);
  if (!sigvalid)
    throw Ehr::SignatureException("Invalid owner");
}

/*----------------------------------------------------------------------------*/

void ProviderI::createInterProviderRequest(const std::string& name,
    Ice::CommunicatorPtr communicator, Ehr::InterProviderPrx& proxy)
throw(Ehr::EhrException)
{
  std::string provaddr = m_data->getConfiguration()->getProviderEndpoint(name);
  if (provaddr.empty())
  {
    LOG4CXX_WARN(logger, "Provider \"" << name << "\" is unknown");
    throw Ehr::EhrException("The provider of the patient isn't known");
  }

  try
  {
    Ice::ObjectPrx base = communicator->stringToProxy("InterProvider:" + provaddr);
    LOG4CXX_DEBUG(logger, "Interprovider request to: \"InterProvider:"
        << provaddr << "\"");
    if (!(proxy = Ehr::InterProviderPrx::checkedCast(base)))
    {
      LOG4CXX_ERROR(logger, "Invalid endpoint of provider \"" << name << "\"");
      throw Ehr::EhrException("The patient's provider connection is invalid state");
    }
  }
  catch (const Ice::Exception &ex)
  {
    LOG4CXX_ERROR(logger, "Unable to connect to provider: " << name
        << " (" << provaddr << ")");
    throw Ehr::EhrException("Unable to connect to the provider of the patient");
  }
}

/* vim: set et sw=2 ts=2: */
