/**
 * $Id: provideri.h 146 2010-01-21 01:02:32Z l0728348 $
 *
 * Copyright 2009
 *
 * @author Christian, Bernhard, Manuel / Gruppe 1
 * @brief Implementation of Client to Provider Interface
 */

#ifndef EHRI_H
#define EHRI_H

#include "datamanager.h"
#include <ehr.h>
#include <interprovider.h>

/**
 * Implementation of the Provider interface
 */
class ProviderI
: virtual public Ehr::Provider
{
  public:
    /**
     * Standard exception to throw at some kind of internal server error
     */
    class internal_error
    : public Ehr::EhrException
    {
      public:
        /**
         * Default exception ctor
         * @param msg message to pass along. Defaults if empty.
         */
        internal_error(const std::string& msg = "")
        {
          what = (msg == "") ? "Internal server error" : msg;
        }
    };

    /**
     * Ctor (empty)
     */
    ProviderI(DataManager *data)
      : m_data(data)
    {}

    /**
     * Dtor (empty)
     */
    virtual ~ProviderI()
    {}

    /**
     * handler for createPrescription-request
     * see ice files for parameters
     * @throw Ehr::EhrException on error
     * @throw internal_error on internal error
     */
    virtual void createPrescription(const ::Ehr::CreatePrescriptionRequestPtr&,
        const ::Ehr::Signature&, const ::Ehr::Signature&, const Ice::Current&)
      throw(internal_error, Ehr::EhrException);

    /**
     * handler for consumePrescription-request
     * see ice files for parameters
     * @throw Ehr::EhrException on error
     * @throw internal_error on internal error
     */
    virtual void consumePrescription(const ::Ehr::ConsumePrescriptionRequestPtr&,
        const ::Ehr::Signature&, const ::Ehr::Signature&, const Ice::Current&)
      throw(internal_error, Ehr::EhrException);

    /**
     * handler for listDocuments-request
     * see ice files for parameters
     * @throw Ehr::EhrException on error
     * @throw internal_error on internal error
     */
    virtual ::Ehr::DocumentList listDocuments(const ::Ehr::ListDocumentsRequestPtr&,
        const ::Ehr::Signature&, const ::Ehr::Signature&, const Ice::Current&)
      throw(internal_error, Ehr::EhrException);

    /**
     * handler for findDocuments-request
     * see ice files for parameters
     * @throw Ehr::EhrException on error
     * @throw internal_error on internal error
     */
    virtual ::Ehr::DocumentList findDocuments(const ::Ehr::FindDocumentsRequestPtr&,
        const ::Ehr::Signature&, const ::Ehr::Signature&, const Ice::Current&)
      throw(internal_error, Ehr::EhrException);

    /**
     * handler for setDefaultAccess-request
     * see ice files for parameters
     * @throw Ehr::EhrException on error
     * @throw internal_error on internal error
     */
    virtual void setDefaultAccess(const ::Ehr::SetDefaultAccessRequestPtr&,
        const ::Ehr::Signature&, const Ice::Current&)
      throw(internal_error, Ehr::EhrException);

    /**
     * handler for createPermission-request
     * see ice files for parameters
     * @throw Ehr::EhrException on error
     * @throw internal_error on internal error
     */
    virtual void createPermission(const ::Ehr::CreatePermissionRequestPtr&,
        const ::Ehr::Signature&, const Ice::Current&)
      throw(internal_error, Ehr::EhrException);

    /**
     * handler for listPermissions-request
     * see ice files for parameters
     * @throw Ehr::EhrException on error
     * @throw internal_error on internal error
     */
    virtual ::Ehr::Permissions listPermissions(const ::Ehr::ListPermissionsRequestPtr&,
        const ::Ehr::Signature&, const Ice::Current&)
      throw(internal_error, Ehr::EhrException);

    /**
     * handler for removePermission-request
     * see ice files for parameters
     * @throw Ehr::EhrException on error
     * @throw internal_error on internal error
     */
    virtual void removePermission(const ::Ehr::RemovePermissionRequestPtr&,
        const ::Ehr::Signature&, const Ice::Current&)
      throw(internal_error, Ehr::EhrException);

  protected:
    /**
      * Verifies the timestamp and requestor of an owner
      * @param  request request obj ptr
      * @param  requestorSig signature of requestor
      * @throw  Ehr::InvalidAccountException on unknown requestor
      * @throw  Ehr::SignatureException on invalid signature
      * @throw  Ehr::InvalidRequestException on invalid timestamp
      */
    void checkRequest(Ehr::RequestPtr request,
        const Ehr::Signature& requestorSig)
      throw(Ehr::EhrException);

    /**
      * Verifies the owner signature for a request.
      * Only possible if owner is a local user.
      * @param  request request obj ptr
      * @param  ownerSig signature of owner
      * @throw  Ehr::InvalidAccountException on unknown requestor
      * @throw  Ehr::SignatureException on invalid signature
      */
    void checkRequestOwner(Ehr::ThirdPartyRequestPtr request,
        const Ehr::Signature& ownerSig)
      throw(Ehr::EhrException);

    /**
     * create interprovider proxy request object for proxy identified
     * by name returned by parameter
     * @param  name name of provider
     * @param  communicator pointer to current ice communicator
     * @param  proxy output parameter for interprovider proxy object
     * @throw  Ehr::EhrException on error
     */
    void createInterProviderRequest(const std::string& name,
        Ice::CommunicatorPtr communicator, Ehr::InterProviderPrx& proxy)
      throw(Ehr::EhrException);

  private:
    DataManager *m_data;
};

typedef IceUtil::Handle<ProviderI> ProviderIPtr;

#endif

/* vim: set et sw=2 ts=2: */
