/**
 * $Id: configurationtest.cpp 179 2010-01-31 00:32:29Z l0728348 $
 *
 * Copyright 2009
 *
 * @author Christian, Bernhard, Manuel / Gruppe 1
 * @brief  Tests for configuration class
 */

#define BOOST_TEST_MODULE configurationtest
#define BOOST_TEST_DYN_LINK
#include "redirector.h"
#include <boost/test/unit_test.hpp>

#include <string>
#include <utility>
#include "../configuration.h"

#define STRINGIFY(x) #x
#define TOSTRING(x) STRINGIFY(x)

#define TESTCONFFILEBAD "ehrprovider-invalid.conf"

using namespace std;

BOOST_AUTO_TEST_SUITE(configuration)

BOOST_AUTO_TEST_CASE(parse1)
{
  const char *argv[] = {
    TOSTRING(BOOST_TEST_MODULE), "-f" TESTCONFDIR "/" TESTCONFFILE, NULL
  };
  int argc;
  for(argc = 0; argv[argc] != NULL; ++argc);

  Configuration conf(argc, const_cast<char **>(argv));

  BOOST_CHECK_EQUAL(conf.operator!(), false);
  BOOST_CHECK_EQUAL(conf.shouldExit(), false);
  BOOST_CHECK_EQUAL(conf.getLastError().empty(), true);
  BOOST_CHECK_EQUAL(conf.getConfFile(), TESTCONFDIR "/" TESTCONFFILE);
  BOOST_CHECK_EQUAL(conf.getConfDir(), TESTCONFDIR);

  BOOST_CHECK_EQUAL(conf.getConfPath(""), "");
  BOOST_CHECK_EQUAL(conf.getConfPath("/some/abs/path"), "/some/abs/path");
  BOOST_CHECK_EQUAL(conf.getConfPath("some/rel/path"), TESTCONFDIR "/some/rel/path");

  // test commandline settings
  BOOST_CHECK_EQUAL(conf.hasVal("f"), true);
  BOOST_CHECK_EQUAL(conf.val<string>("f"), TESTCONFDIR "/" TESTCONFFILE);
  BOOST_CHECK_THROW(conf.val<unsigned>("f"), Configuration::runtime_error);

  BOOST_CHECK_EQUAL(conf.hasVal("d"), false);
  BOOST_CHECK_THROW(conf.val<string>("d"), Configuration::runtime_error);
  BOOST_CHECK_EQUAL(conf.hasVal("help"), false);
  BOOST_CHECK_THROW(conf.val<string>("help"), Configuration::runtime_error);

  // test some ini settings
  BOOST_CHECK_EQUAL(conf.hasVal("network.endpoint"), true);
  BOOST_CHECK_EQUAL(conf.val<string>("network.endpoint"), "ssl -h localhost -p 3434");
  BOOST_CHECK_EQUAL(conf.hasVal("certificates.providermap"), true);
  BOOST_CHECK_EQUAL(conf.val<string>("certificates.providermap"), "providermap.xml");
  BOOST_CHECK_EQUAL(conf.hasVal("database.poolsize"), true);
  BOOST_CHECK_EQUAL(conf.val<unsigned>("database.poolsize"), 5);

  // test default value
  BOOST_CHECK_EQUAL(conf.hasVal("database.type"), true);
  BOOST_CHECK_EQUAL(conf.val<string>("database.type").empty(), false);
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(noconf)
{
  const char *argv[] = {
    TOSTRING(BOOST_TEST_MODULE),
    "-f/path/to/some/nonexistend/directory/containing/a/conf/file",
    NULL
  };
  int argc;
  for(argc = 0; argv[argc] != NULL; ++argc);

  Configuration conf(argc, const_cast<char **>(argv));

  BOOST_CHECK_EQUAL(conf.operator!(), true);
  BOOST_CHECK_EQUAL(conf.shouldExit(), false);
  BOOST_CHECK_EQUAL(conf.getLastError().empty(), false);
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(invalid_conf)
{
  const char *argv[] = {
    TOSTRING(BOOST_TEST_MODULE), "-f" TESTCONFDIR "/" TESTCONFFILEBAD, NULL
  };
  int argc;
  for(argc = 0; argv[argc] != NULL; ++argc);

  Configuration conf(argc, const_cast<char **>(argv));

  BOOST_CHECK_EQUAL(conf.operator!(), true);
  BOOST_CHECK_EQUAL(conf.shouldExit(), false);
  BOOST_CHECK_EQUAL(conf.getLastError().empty(), false);
  BOOST_CHECK_EQUAL(conf.getConfFile(), TESTCONFDIR "/" TESTCONFFILEBAD);
  BOOST_CHECK_EQUAL(conf.getConfDir(), TESTCONFDIR);
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(conf_is_dir)
{
  const char *argv[] = {
    TOSTRING(BOOST_TEST_MODULE), "-f" TESTCONFDIR, NULL
  };
  int argc;
  for(argc = 0; argv[argc] != NULL; ++argc);

  Configuration conf(argc, const_cast<char **>(argv));

  string dirname = TESTCONFDIR;
  dirname = dirname.substr(0, dirname.find_last_of('/'));

  BOOST_CHECK_EQUAL(conf.operator!(), true);
  BOOST_CHECK_EQUAL(conf.shouldExit(), false);
  BOOST_CHECK_EQUAL(conf.getLastError().empty(), false);
  BOOST_CHECK_EQUAL(conf.getConfFile(), TESTCONFDIR);
  BOOST_CHECK_EQUAL(conf.getConfDir(), dirname);
}

/*----------------------------------------------------------------------------*/


BOOST_AUTO_TEST_CASE(cmdline_help)
{
  const char *argv[] = {
    TOSTRING(BOOST_TEST_MODULE), "-h", NULL
  };
  int argc;
  for(argc = 0; argv[argc] != NULL; ++argc);

  Configuration conf(argc, const_cast<char **>(argv));

  BOOST_CHECK_EQUAL(conf.operator!(), false);
  BOOST_CHECK_EQUAL(conf.getLastError().empty(), true);
  BOOST_CHECK_EQUAL(conf.shouldExit(), true);
  BOOST_CHECK_EQUAL(conf.hasVal("help"), true);
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(cmdline_version)
{
  const char *argv[] = {
    TOSTRING(BOOST_TEST_MODULE), "-v", NULL
  };
  int argc;
  for(argc = 0; argv[argc] != NULL; ++argc);

  Configuration conf(argc, const_cast<char **>(argv));

  BOOST_CHECK_EQUAL(conf.operator!(), false);
  BOOST_CHECK_EQUAL(conf.getLastError().empty(), true);
  BOOST_CHECK_EQUAL(conf.shouldExit(), true);
  BOOST_CHECK_EQUAL(conf.hasVal("version"), true);
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(cmdline_unknown_option)
{
  const char *argv[] = {
    TOSTRING(BOOST_TEST_MODULE), "-s", NULL
  };
  int argc;
  for(argc = 0; argv[argc] != NULL; ++argc);

  Configuration conf(argc, const_cast<char **>(argv));

  BOOST_CHECK_EQUAL(conf.operator!(), true);
  BOOST_CHECK_EQUAL(conf.shouldExit(), false);
  BOOST_CHECK_EQUAL(conf.getLastError().empty(), false);
  BOOST_CHECK_EQUAL(conf.hasVal("s"), false);
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(cmdline_multiple)
{
  const char *argv[] = {
    TOSTRING(BOOST_TEST_MODULE), "-d", "-d", NULL
  };
  int argc;
  for(argc = 0; argv[argc] != NULL; ++argc);

  Configuration conf(argc, const_cast<char **>(argv));

  BOOST_CHECK_EQUAL(conf.operator!(), true);
  BOOST_CHECK_EQUAL(conf.shouldExit(), false);
  BOOST_CHECK_EQUAL(conf.getLastError().empty(), false);
  BOOST_CHECK_EQUAL(conf.hasVal("d"), true);
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(override)
{
  const char *argv[] = {
    TOSTRING(BOOST_TEST_MODULE), "-f" TESTCONFDIR "/" TESTCONFFILE,
    "-onetwork.endpoint=\"abc def gji\"",
    "-odatabase.dsn=abc123",
    "-odatabase.poolsize=10",
    NULL
  };
  int argc;
  for(argc = 0; argv[argc] != NULL; ++argc);

  Configuration conf(argc, const_cast<char **>(argv));

  BOOST_CHECK_EQUAL(conf.operator!(), false);
  BOOST_CHECK_EQUAL(conf.shouldExit(), false);
  BOOST_CHECK_EQUAL(conf.getLastError().empty(), true);
  BOOST_CHECK_EQUAL(conf.getConfFile(), TESTCONFDIR "/" TESTCONFFILE);
  BOOST_CHECK_EQUAL(conf.getConfDir(), TESTCONFDIR);

  BOOST_CHECK_EQUAL(conf.hasVal("network.endpoint"), true);
  BOOST_CHECK_EQUAL(conf.val<string>("network.endpoint"), "abc def gji");
  BOOST_CHECK_EQUAL(conf.hasVal("database.dsn"), true);
  BOOST_CHECK_EQUAL(conf.val<string>("database.dsn"), "abc123");
  BOOST_CHECK_EQUAL(conf.hasVal("database.poolsize"), true);
  BOOST_CHECK_EQUAL(conf.val<unsigned>("database.poolsize"), 10);
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(override_unknown)
{
  const char *argv[] = {
    TOSTRING(BOOST_TEST_MODULE), "-f" TESTCONFDIR "/" TESTCONFFILE,
    "-onetwork.xyz=notknown", NULL
  };
  int argc;
  for(argc = 0; argv[argc] != NULL; ++argc);

  Configuration conf(argc, const_cast<char **>(argv));

  BOOST_CHECK_EQUAL(conf.operator!(), true);
  BOOST_CHECK_EQUAL(conf.shouldExit(), false);
  BOOST_CHECK_EQUAL(conf.getLastError().empty(), false);
  BOOST_CHECK_EQUAL(conf.getConfFile(), TESTCONFDIR "/" TESTCONFFILE);
  BOOST_CHECK_EQUAL(conf.getConfDir(), TESTCONFDIR);

  BOOST_CHECK_EQUAL(conf.hasVal("network.xyz"), false);
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(override_badvalue)
{
  const char *argv[] = {
    TOSTRING(BOOST_TEST_MODULE), "-f" TESTCONFDIR "/" TESTCONFFILE,
    "-odatabase.poolsize=stringbutnumberexpected", NULL
  };
  int argc;
  for(argc = 0; argv[argc] != NULL; ++argc);

  Configuration conf(argc, const_cast<char **>(argv));

  BOOST_CHECK_EQUAL(conf.operator!(), true);
  BOOST_CHECK_EQUAL(conf.shouldExit(), false);
  BOOST_CHECK_EQUAL(conf.getLastError().empty(), false);
  BOOST_CHECK_EQUAL(conf.getConfFile(), TESTCONFDIR "/" TESTCONFFILE);
  BOOST_CHECK_EQUAL(conf.getConfDir(), TESTCONFDIR);

  BOOST_CHECK_EQUAL(conf.hasVal("database.poolsize"), false);
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(override_badsyntax1)
{
  const char *argv[] = {
    TOSTRING(BOOST_TEST_MODULE), "-f" TESTCONFDIR "/" TESTCONFFILE,
    "-okeyvalueinonestring", NULL
  };
  int argc;
  for(argc = 0; argv[argc] != NULL; ++argc);

  Configuration conf(argc, const_cast<char **>(argv));

  BOOST_CHECK_EQUAL(conf.operator!(), true);
  BOOST_CHECK_EQUAL(conf.shouldExit(), false);
  BOOST_CHECK_EQUAL(conf.getLastError().empty(), false);
  BOOST_CHECK_EQUAL(conf.getConfFile(), TESTCONFDIR "/" TESTCONFFILE);
  BOOST_CHECK_EQUAL(conf.getConfDir(), TESTCONFDIR);
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(override_badsyntax2)
{
  const char *argv[] = {
    TOSTRING(BOOST_TEST_MODULE), "-f" TESTCONFDIR "/" TESTCONFFILE,
    "-okey=", NULL
  };
  int argc;
  for(argc = 0; argv[argc] != NULL; ++argc);

  Configuration conf(argc, const_cast<char **>(argv));

  BOOST_CHECK_EQUAL(conf.operator!(), true);
  BOOST_CHECK_EQUAL(conf.shouldExit(), false);
  BOOST_CHECK_EQUAL(conf.getLastError().empty(), false);
  BOOST_CHECK_EQUAL(conf.getConfFile(), TESTCONFDIR "/" TESTCONFFILE);
  BOOST_CHECK_EQUAL(conf.getConfDir(), TESTCONFDIR);
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(override_badsyntax3)
{
  const char *argv[] = {
    TOSTRING(BOOST_TEST_MODULE), "-f" TESTCONFDIR "/" TESTCONFFILE,
    "-o=value", NULL
  };
  int argc;
  for(argc = 0; argv[argc] != NULL; ++argc);

  Configuration conf(argc, const_cast<char **>(argv));

  BOOST_CHECK_EQUAL(conf.operator!(), true);
  BOOST_CHECK_EQUAL(conf.shouldExit(), false);
  BOOST_CHECK_EQUAL(conf.getLastError().empty(), false);
  BOOST_CHECK_EQUAL(conf.getConfFile(), TESTCONFDIR "/" TESTCONFFILE);
  BOOST_CHECK_EQUAL(conf.getConfDir(), TESTCONFDIR);
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(override_badsyntax4)
{
  const char *argv[] = {
    TOSTRING(BOOST_TEST_MODULE), "-f" TESTCONFDIR "/" TESTCONFFILE,
    "-o=", NULL
  };
  int argc;
  for(argc = 0; argv[argc] != NULL; ++argc);

  Configuration conf(argc, const_cast<char **>(argv));

  BOOST_CHECK_EQUAL(conf.operator!(), true);
  BOOST_CHECK_EQUAL(conf.shouldExit(), false);
  BOOST_CHECK_EQUAL(conf.getLastError().empty(), false);
  BOOST_CHECK_EQUAL(conf.getConfFile(), TESTCONFDIR "/" TESTCONFFILE);
  BOOST_CHECK_EQUAL(conf.getConfDir(), TESTCONFDIR);
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(parse2)
{
  const char *argv[] = {
    TOSTRING(BOOST_TEST_MODULE), "-f" TESTCONFDIR "/" TESTCONFFILE, NULL
  };
  int argc;
  for(argc = 0; argv[argc] != NULL; ++argc);

  Configuration conf(argc, const_cast<char **>(argv));

  BOOST_CHECK_EQUAL(conf.operator!(), false);
  BOOST_CHECK_EQUAL(conf.shouldExit(), false);
  BOOST_CHECK_EQUAL(conf.getLastError().empty(), true);

  BOOST_CHECK_EQUAL(conf.loadCertificates(), true);
  BOOST_CHECK_EQUAL(conf.getMyName(), "provider1");
  BOOST_CHECK_EQUAL(conf.getProviderCert("provider2"), TESTCONFDIR "/certs/provider/provider2.pem");
  BOOST_CHECK_EQUAL(conf.getProviderEndpoint("provider2"), "ssl -h localhost -p 3435");
  BOOST_CHECK_EQUAL(conf.getUserCert("pat1"), TESTCONFDIR "/certs/user/pat1@provider1.pem");
  BOOST_CHECK_EQUAL(conf.getUserCert("pat1", "provider1"), TESTCONFDIR "/certs/user/pat1@provider1.pem");

  BOOST_CHECK_EQUAL(conf.getProviderCert("provider_notknown"), "");
  BOOST_CHECK_EQUAL(conf.getUserCert("pat_notknown"), "");
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(missing_rootca)
{
  const char *argv[] = {
    TOSTRING(BOOST_TEST_MODULE), "-f" TESTCONFDIR "/" TESTCONFFILE,
    "-ocertificates.rootca=/does/not/exist",
    NULL
  };
  int argc;
  for(argc = 0; argv[argc] != NULL; ++argc);

  Configuration conf(argc, const_cast<char **>(argv));

  BOOST_CHECK_EQUAL(conf.operator!(), false);
  BOOST_CHECK_EQUAL(conf.shouldExit(), false);
  BOOST_CHECK_EQUAL(conf.getLastError().empty(), true);

  BOOST_CHECK_EQUAL(conf.loadCertificates(), false);
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(missing_providermap)
{
  const char *argv[] = {
    TOSTRING(BOOST_TEST_MODULE), "-f" TESTCONFDIR "/" TESTCONFFILE,
    "-ocertificates.providermap=/does/not/exist",
    NULL
  };
  int argc;
  for(argc = 0; argv[argc] != NULL; ++argc);

  Configuration conf(argc, const_cast<char **>(argv));

  BOOST_CHECK_EQUAL(conf.operator!(), false);
  BOOST_CHECK_EQUAL(conf.shouldExit(), false);
  BOOST_CHECK_EQUAL(conf.getLastError().empty(), true);

  BOOST_CHECK_EQUAL(conf.loadCertificates(), false);
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(corrupt_mycert)
{
  const char *argv[] = {
    TOSTRING(BOOST_TEST_MODULE), "-f" TESTCONFDIR "/" TESTCONFFILE,
    "-ocertificates.mycert=" TESTCONFFILE,
    NULL
  };
  int argc;
  for(argc = 0; argv[argc] != NULL; ++argc);

  Configuration conf(argc, const_cast<char **>(argv));

  BOOST_CHECK_EQUAL(conf.operator!(), false);
  BOOST_CHECK_EQUAL(conf.shouldExit(), false);
  BOOST_CHECK_EQUAL(conf.getLastError().empty(), true);

  BOOST_CHECK_EQUAL(conf.loadCertificates(), false);
}

BOOST_AUTO_TEST_SUITE_END()

/* vim: set et sw=2 ts=2: */
