/**
 * $Id: interprovidertest.cpp 178 2010-01-30 22:56:33Z l0728348 $
 *
 * Copyright 2009
 *
 * @author Christian, Bernhard, Manuel
 * @brief Tests for providerinterface
 */

#define BOOST_TEST_MODULE interprovidertest
#define BOOST_TEST_DYN_LINK
#include "redirector.h"
#include <boost/test/unit_test.hpp>
#include <boost/test/utils/runtime/env/variable.hpp>

#include <Ice/Ice.h>
#include <Ice/Plugin.h>
#include <string>
#include <vector>
#include <string.h>

#include "providertestutility.h"
#include "../datamanager.h"
#include "../configuration.h"
#include "../provideri.h"
#include "../interprovideri.h"
#include "../security.h"
#include "../util.h"
#include <ehr.h>

using std::string;
using std::vector;

#define STRINGIFY(x) #x
#define TOSTRING(x) STRINGIFY(x)

using namespace Ehr;

/**
 * Fixture to set up an adapter for interprovider as well as an
 * unregistered local providerI instance
 */
struct InterProviderFixture
: public ProviderFixture
{
  InterProviderFixture()
    : ProviderFixture()
  {
    /* setup second instance of data manager, so that it has a different name */
    /* create argv inside std::vector */
    std::vector<std::string>argvvec;
    argvvec.push_back(TOSTRING(BOOST_TEST_MODULE));
    argvvec.push_back("-f" TESTCONFDIR "/" TESTCONFFILE);
    boost::optional<std::string> dbdsn;
    boost::runtime::env::get("TEST_DBDSN", dbdsn);
    if (dbdsn)
      argvvec.push_back("-odatabase.dsn=\"" + dbdsn.get() + "\"");
    argvvec.push_back("-onetwork.endpoint=\"" + m_conf->getProviderEndpoint("provider2") + "\"");
    argvvec.push_back("-ocertificates.mycert=certs/provider/provider2.pem");
    argvvec.push_back("-ocertificates.mykey=certs/provider/provider2.pem");

    /* convert to c-style const char**. this is a bit hackish */
    unsigned argc = argvvec.size();
    char **argv = new char*[argc + 2];
    for(unsigned i = 0; i < argc; ++i)
      argv[i] = strdup(argvvec[i].c_str());
    argv[argc] = NULL;

    // conf setup
    m_conf2 = new Configuration(argc, const_cast<char **>(argv));
    BOOST_ASSERT(m_conf2->loadCertificates());

    m_data2 = new DataManager(m_conf2);
    if (!*m_data2 || !m_data2->connect())
      BOOST_FAIL("could not setup data manager");
    m_data2->setCommunicator(ic);

    // create provider2
    prov2 = new ProviderI(m_data2);

    // get test pems of provider2
    pat3_pem_file = m_conf2->getUserCert("pat3");
    pat4_pem_file = m_conf2->getUserCert("pat4");
    phy3_pem_file = m_conf2->getUserCert("phy3");
    phy4_pem_file = m_conf2->getUserCert("phy4");
    pharm3_pem_file = m_conf2->getUserCert("pharm3");
    pharm4_pem_file = m_conf2->getUserCert("pharm4");

    // setup users of provider2
    m_data2->createUser("pat3", Ehr::PATIENT);
    m_data2->createUser("pat4", Ehr::PATIENT);
    m_data2->createUser("phy3", Ehr::PHYSICAN);
    m_data2->createUser("phy4", Ehr::PHYSICAN);
    m_data2->createUser("pharm3", Ehr::PHARMACIST);
    m_data2->createUser("pharm4", Ehr::PHARMACIST);

    // setup interprovider adapter for provider1
    m_interprovider = new InterProviderI(m_data);

    ic->getPluginManager()->initializePlugins();

    Ice::PropertiesPtr props = ic->getProperties();
    props->setProperty("IceSSL.CertAuthFile",
        m_conf->getConfPath(m_conf->val<string>("certificates.rootca")));
    props->setProperty("IceSSL.CertFile",
        m_conf->getConfPath(m_conf->val<string>("certificates.mycert")));
    props->setProperty("IceSSL.KeyFile",
        m_conf->getConfPath(m_conf->val<string>("certificates.mykey")));

    m_providerAdapter = ic->createObjectAdapterWithEndpoints("ProviderAdapter",
        m_conf->val<string>("network.endpoint"));
    m_providerAdapter->add(m_interprovider, ic->stringToIdentity("InterProvider"));
    m_providerAdapter->activate();
  }

  virtual ~InterProviderFixture()
  {
    m_providerAdapter->deactivate();
    delete m_interprovider;
    delete prov2;
    delete m_data2;
    delete m_conf2;
  }

  ProviderI *prov2;
  Ice::ObjectAdapterPtr m_providerAdapter;
  InterProviderI *m_interprovider;
  Configuration *m_conf2;
  DataManager *m_data2;
  string pat3_pem_file;
  string pat4_pem_file;
  string phy3_pem_file;
  string phy4_pem_file;
  string pharm3_pem_file;
  string pharm4_pem_file;
};

/*----------------------------------------------------------------------------*/

BOOST_FIXTURE_TEST_SUITE(InterProviderTestSuite, InterProviderFixture)

BOOST_AUTO_TEST_CASE(list_documents)
{
  insertDocuments();
  Ice::Current c = Ice::Current();
  c.adapter = m_providerAdapter;

  string requestor = phy3_pem_file;
  string owner = pat1_pem_file;

  DocumentList l = listDocuments(prov2, requestor, owner, c);
  BOOST_CHECK_EQUAL(l.size(), NUM_TEST_DOCS_FIRST_PAT);
  for (unsigned i = 0; i < NUM_TEST_DOCS_FIRST_PAT; i++)
  {
    BOOST_CHECK_EQUAL(l[i].document.encryptedData[0], i);
    BOOST_CHECK_EQUAL(l[i].document.encryptedKey[0], i);
    BOOST_CHECK_EQUAL(l[i].document.initialVector[0], i);
  }
  removeDocuments();
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(find_documents)
{
  insertDocuments();
  Ice::Current c = Ice::Current();
  c.adapter = m_providerAdapter;

  string requestor = phy3_pem_file;
  string owner = pat1_pem_file;

  // find all documents
  checkFindDocuments(prov2, NUM_TEST_DOCS_FIRST_PAT,
      createFindDocumentsRequest(requestor, owner), requestor, owner, c);

  // search for consumed (no matches)
  checkFindDocuments(prov2, NUM_TEST_DOCS_FIRST_PAT,
      createFindDocumentsRequest(phy3_pem_file, pat1_pem_file,
        -1, -1, DOCCONSUMEDPRESCRIPTION), requestor, owner, c);

  // search for normal prescriptions (all match)
  checkFindDocuments(prov2, NUM_TEST_DOCS_FIRST_PAT,
      createFindDocumentsRequest(phy3_pem_file, pat1_pem_file,
        -1, -1, DOCPRESCRIPTION), requestor, owner, c);

  // search timespan (all match)
  checkFindDocuments(prov2, NUM_TEST_DOCS_FIRST_PAT,
      createFindDocumentsRequest(phy3_pem_file, pat1_pem_file,
        Util::getTime() - 100000, Util::getTime() + 100000),
      requestor, owner, c);

  removeDocuments();
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(consume_prescr)
{
  Ice::Current c = Ice::Current();
  c.adapter = m_providerAdapter;

  string requestor = phy3_pem_file;
  string owner = pat1_pem_file;

  /* create documents */
  for (unsigned i = 0; i < NUM_TEST_DOCS_FIRST_PAT; i++)
  {
    Ehr::CreatePrescriptionRequestPtr r = createCreatePrescriptionRequest(
        requestor, owner, documents[i]);
    prov2->createPrescription(r,
        createSignatureForRequest(requestor, r),
        createSignatureForRequest(owner, r), c);
  }

  /* fetch all documents of owner */
  DocumentList l = listDocuments(prov2, requestor, owner, c);
  BOOST_CHECK_EQUAL(l.size(), NUM_TEST_DOCS_FIRST_PAT);

  /* consume all documents of owner */
  for (DocumentList::iterator i = l.begin(); i != l.end(); i++)
  {
    ConsumePrescriptionRequestPtr consumeReq = createConsumePrescriptionRequest(
        requestor, owner, i->id);
    prov2->consumePrescription(consumeReq,
        createSignatureForRequest(requestor, consumeReq),
        createSignatureForRequest(owner, consumeReq),
        c);
  }

  /* check if no document got lost */
  DocumentList l2 = listDocuments(prov2, requestor, owner, c);
  BOOST_CHECK_EQUAL(l2.size(), NUM_TEST_DOCS_FIRST_PAT);

  removeDocuments();
}

BOOST_AUTO_TEST_SUITE_END()
