/**
 * $Id: providertest.cpp 178 2010-01-30 22:56:33Z l0728348 $
 *
 * Copyright 2009
 *
 * @author Christian, Bernhard, Manuel
 * @brief Tests for providerinterface
 */

#define BOOST_TEST_MODULE providertest
#define BOOST_TEST_DYN_LINK
#include "redirector.h"
#include <boost/test/unit_test.hpp>
#include <boost/test/utils/runtime/env/variable.hpp>


#include <Ice/Ice.h>
#include <string>
#include <vector>
#include <string.h>

#include "providertestutility.h"
#include "../datamanager.h"
#include "../configuration.h"
#include "../provideri.h"
#include "../security.h"
#include "../util.h"
#include <ehr.h>

using std::string;
using std::vector;

#define STRINGIFY(x) #x
#define TOSTRING(x) STRINGIFY(x)


using namespace Ehr;

/*----------------------------------------------------------------------------*/

BOOST_FIXTURE_TEST_SUITE(ProviderTestSuite, ProviderFixture)

BOOST_AUTO_TEST_CASE(test_invalid_timestamp)
{
  ::Ehr::ListDocumentsRequestPtr r = createListDocumentsRequest(pat1_pem_file, pat1_pem_file);
  r->when.msecs = 42;
  BOOST_REQUIRE_THROW(
  prov->listDocuments(r,
                      createSignatureForRequest(pat1_pem_file, r),
                      createSignatureForRequest(pat1_pem_file, r),
                      Ice::Current()),
                      Ehr::InvalidRequestException);
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(test_invalid_signature_owner)
{
  ::Ehr::ListDocumentsRequestPtr r = createListDocumentsRequest(pat1_pem_file, pat1_pem_file);
  BOOST_REQUIRE_THROW(
  prov->listDocuments(r,
                      Ehr::Signature(),
                      createSignatureForRequest(pat1_pem_file, r),
                      Ice::Current()),
                      Ehr::SignatureException);
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(test_invalid_signature_requestor)
{
  ::Ehr::ListDocumentsRequestPtr r = createListDocumentsRequest(pat1_pem_file, pat1_pem_file);
  BOOST_REQUIRE_THROW(
  prov->listDocuments(r,
                      createSignatureForRequest(pat1_pem_file, r),
                      Ehr::Signature(),
                      Ice::Current()),
                      Ehr::SignatureException);
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(test_list_docs)
{
  {
    Ehr::DocumentList l = listDocuments(prov, pat1_pem_file, pat1_pem_file);
    BOOST_CHECK(l.empty());
  }

  // check if we aren't allowed to view others docs
  {
    ListDocumentsRequestPtr r = createListDocumentsRequest(pat1_pem_file, pat2_pem_file);
    BOOST_CHECK_THROW(
      prov->listDocuments(r,
                          createSignatureForRequest(pat1_pem_file, r),
                          createSignatureForRequest(pat2_pem_file, r),
                          Ice::Current()),
      PermissionDeniedException
    );
  }
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(insert_docs)
{
  insertDocuments();
  // check if inserted documents are there
  Ehr::DocumentList l = listDocuments(prov, pat1_pem_file, pat1_pem_file);
  BOOST_CHECK_EQUAL(l.size(), NUM_TEST_DOCS_FIRST_PAT);
  for (unsigned i=0; i<NUM_TEST_DOCS_FIRST_PAT; i++) {
    // check contents of each doc
    BOOST_CHECK_EQUAL(l[i].document.encryptedData[0], i);
    BOOST_CHECK_EQUAL(l[i].document.encryptedKey[0], i);
    BOOST_CHECK_EQUAL(l[i].document.initialVector[0], i);
  }
  removeDocuments();
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(consume_presrc)
{
  insertDocuments();
  // get a list of all documents and consume them
  DocumentList l = listDocuments(prov, pat1_pem_file, pat1_pem_file);
  for (DocumentList::iterator i = l.begin(); i != l.end(); i++) {
    ConsumePrescriptionRequestPtr consumeReq =
    createConsumePrescriptionRequest(pharm1_pem_file, pat1_pem_file, i->id);
    prov->consumePrescription(consumeReq,
                              createSignatureForRequest(pharm1_pem_file, consumeReq),
                              createSignatureForRequest(pat1_pem_file, consumeReq),
                              Ice::Current());
  }
  // check if all are there
  DocumentList l2 = listDocuments(prov, pat1_pem_file, pat1_pem_file);
  BOOST_CHECK_EQUAL(l2.size(), NUM_TEST_DOCS_FIRST_PAT);
  removeDocuments();
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(consume_presrc_invalid)
{
  insertDocuments();
  // check if a patient can't consume
  Ice::Long docId = listDocuments(prov, pat1_pem_file, pat1_pem_file)[0].id;
  ConsumePrescriptionRequestPtr consumeReq =
    createConsumePrescriptionRequest(pat1_pem_file, pat1_pem_file, docId);
  BOOST_CHECK_THROW(
    prov->consumePrescription(consumeReq,
                            createSignatureForRequest(pat1_pem_file, consumeReq),
                            createSignatureForRequest(pat1_pem_file, consumeReq),
                            Ice::Current()),
                    PermissionDeniedException
                    );
  removeDocuments();
}


/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(find_documents)
{
  insertDocuments();

  // find all documents
  checkFindDocuments(prov, NUM_TEST_DOCS_FIRST_PAT,
                     createFindDocumentsRequest(pat1_pem_file, pat1_pem_file));

  // search for consumed (no matches)
  checkFindDocuments(prov, NUM_TEST_DOCS_FIRST_PAT,
                     createFindDocumentsRequest(pat1_pem_file, pat1_pem_file,
                     -1, -1, DOCCONSUMEDPRESCRIPTION));

  // search for normal prescriptions (all match)
  checkFindDocuments(prov, NUM_TEST_DOCS_FIRST_PAT,
                     createFindDocumentsRequest(pat1_pem_file, pat1_pem_file,
                                                -1, -1, DOCPRESCRIPTION));

  // search timespan (all match)
  checkFindDocuments(prov, NUM_TEST_DOCS_FIRST_PAT,
                     createFindDocumentsRequest(pat1_pem_file, pat1_pem_file,
                                                Util::getTime() - 100000,
                                                Util::getTime() + 100000));
  removeDocuments();
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(default_access)
{
  insertDocuments();
  // no permissions
  Ice::Long docId = listDocuments(prov, pat1_pem_file, pat1_pem_file)[0].id;
  {
    ListPermissionsRequestPtr r = createListPermissionsRequest(pat1_pem_file, docId);
    Permissions p = prov->listPermissions(r, createSignatureForRequest(pat1_pem_file, r),
                                          Ice::Current());
    BOOST_CHECK_EQUAL(p.defaultAccess,
                      (m_conf->val<std::string>("misc.defaultacl") == "allow") ? ALLOW : DENY);
  }

  // set default to allow
  {
    SetDefaultAccessRequestPtr r2 = createSetDefaultAccessRequest(pat1_pem_file, docId, ALLOW);
    prov->setDefaultAccess(r2, createSignatureForRequest(pat1_pem_file, r2), Ice::Current());
    ListPermissionsRequestPtr r = createListPermissionsRequest(pat1_pem_file, docId);
    Permissions p = prov->listPermissions(r, createSignatureForRequest(pat1_pem_file, r),
                                          Ice::Current());
    BOOST_CHECK_EQUAL(p.defaultAccess, ALLOW);
  }

  // set default to deny
  {
    SetDefaultAccessRequestPtr r2 = createSetDefaultAccessRequest(pat1_pem_file, docId, DENY);
    prov->setDefaultAccess(r2, createSignatureForRequest(pat1_pem_file, r2), Ice::Current());
    ListPermissionsRequestPtr r = createListPermissionsRequest(pat1_pem_file, docId);
    Permissions p = prov->listPermissions(r, createSignatureForRequest(pat1_pem_file, r),
                                          Ice::Current());
    BOOST_CHECK_EQUAL(p.defaultAccess, DENY);
  }

  // check if access really is denied to others
  FindDocumentsRequestPtr findReq1 = createFindDocumentsRequest(pharm1_pem_file, pat1_pem_file,
                                                               -1, -1, DOCANY, docId);
  checkFindDocuments(prov, 0, findReq1, pharm1_pem_file);

  // check if access is still allowed for owner
  FindDocumentsRequestPtr findReq2 = createFindDocumentsRequest(pat1_pem_file, pat1_pem_file,
                                                               -1, -1, DOCANY, docId);
  checkFindDocuments(prov, 1, findReq2);

  removeDocuments();
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_CASE(single_permission)
{
  insertDocuments();
  Ice::Long docId = listDocuments(prov, pat1_pem_file, pat1_pem_file)[0].id;
  // create a deny permission
  CreatePermissionRequestPtr r =
    createCreatePermissionRequest(pat1_pem_file, docId,
                                  getAccountIdentifierFromFile(phy1_pem_file),
                                  DENY);
  prov->createPermission(r, createSignatureForRequest(pat1_pem_file, r), Ice::Current());

  // check if access is denied
  {
    FindDocumentsRequestPtr findReq = createFindDocumentsRequest(phy1_pem_file, pat1_pem_file,
                                                                  -1, -1, DOCANY, docId);
    checkFindDocuments(prov, 0, findReq, phy1_pem_file);
  }

  ListPermissionsRequestPtr lpr = createListPermissionsRequest(pat1_pem_file, docId);
  Permissions p = prov->listPermissions(lpr, createSignatureForRequest(pat1_pem_file, lpr), Ice::Current());
  BOOST_CHECK_EQUAL(p.acl.size(), 1);
  BOOST_CHECK_EQUAL(p.acl[0].access, DENY);
  BOOST_CHECK(p.acl[0].account == getAccountIdentifierFromFile(phy1_pem_file));

  RemovePermissionRequestPtr rpr = createRemovePermissionRequest(pat1_pem_file, docId, p.acl[0].id);
  prov->removePermission(rpr, createSignatureForRequest(pat1_pem_file, rpr), Ice::Current());

  // check if access is allowed again
  {
    FindDocumentsRequestPtr findReq = createFindDocumentsRequest(phy1_pem_file, pat1_pem_file,
                                                                  -1, -1, DOCANY, docId);
    checkFindDocuments(prov, 1, findReq, phy1_pem_file);
  }

  ListPermissionsRequestPtr lpr2 = createListPermissionsRequest(pat1_pem_file, docId);
  Permissions p2 = prov->listPermissions(lpr2, createSignatureForRequest(pat1_pem_file, lpr2), Ice::Current());
  BOOST_CHECK_EQUAL(p2.acl.size(), 0);

  removeDocuments();
}

/*----------------------------------------------------------------------------*/

BOOST_AUTO_TEST_SUITE_END()

/* vim: set et sw=2 ts=2: */
