/**
 * $Id: providertestutility.h 178 2010-01-30 22:56:33Z l0728348 $
 *
 * Copyright 2009
 *
 * @author Christian, Bernhard, Manuel
 * @brief Test utility for providerinterface
 */

#ifndef PROVIDERTESTUTILITY_H
#define PROVIDERTESTUTILITY_H

#include "redirector.h"
#include <boost/test/unit_test.hpp>
#include <boost/test/utils/runtime/env/variable.hpp>

#include <Ice/Ice.h>
#include <string>
#include <vector>
#include <string.h>

#include "../datamanager.h"
#include "../configuration.h"
#include "../provideri.h"
#include "../security.h"
#include "../util.h"
#include <ehr.h>

using std::string;
using std::vector;

#define STRINGIFY(x) #x
#define TOSTRING(x) STRINGIFY(x)

using namespace Ehr;

/**
 * Misc test data
 */
const unsigned NUM_TEST_DOCS = 5;
const unsigned NUM_TEST_DOCS_FIRST_PAT = 3;

/*----------------------------------------------------------------------------*/

Ehr::AccountIdentifier getAccountIdentifierFromFile(const std::string& filename)
{
  std::string commonName = Security::instance().getCommonName(filename) ;
  // strip string by '@' as defined in the standard
  std::size_t pos = commonName.find("@");
  if (pos == std::string::npos) {
    throw ::Ehr::EhrException( std::string("Invalid CN field in PEM file ")+filename);
  }
  ::Ehr::AccountIdentifier ident;
  ident.user = commonName.substr(0, pos);
  ident.provider = commonName.substr(pos+1, std::string::npos);
  return ident;
}

/*----------------------------------------------------------------------------*/

/**
 * Sets data for a Request obj
 */
void setupRequest(RequestPtr r, string requestorPemFile) {
  r->when.msecs = Util::getTime();
  r->requestor = getAccountIdentifierFromFile(requestorPemFile);
}

/**
 * Sets data for a ThirdPartyRequest obj
 */
void setupThirdPartyRequest(ThirdPartyRequestPtr r, string requestorPemFile, string ownerPemFile) {
  setupRequest(r, requestorPemFile);
  r->owner = getAccountIdentifierFromFile(ownerPemFile);
}

/**
 * Returns a valid list document request
 */
::Ehr::ListDocumentsRequestPtr
createListDocumentsRequest(string requestorPemFile, string ownerPemFile)
{
  ::Ehr::ListDocumentsRequestPtr r = new ::Ehr::ListDocumentsRequest;
  setupThirdPartyRequest(r, requestorPemFile, ownerPemFile);
  return r;
}

/**
 * Returns a valid create prescription request
 */
Ehr::CreatePrescriptionRequestPtr
createCreatePrescriptionRequest(string requestorPemFile, string ownerPemFile,
                                EncryptedDocument doc = Ehr::EncryptedDocument())
{
  CreatePrescriptionRequestPtr r = new ::Ehr::CreatePrescriptionRequest;
  setupThirdPartyRequest(r, requestorPemFile, ownerPemFile);
  r->prescription = doc;
  return r;
}

/**
 * Returns a consume prescr request
 */
ConsumePrescriptionRequestPtr
createConsumePrescriptionRequest(string requestorPemFile, string ownerPemFile,
                                 Ice::Long documentToConsume,
                                 EncryptedDocument doc = Ehr::EncryptedDocument())
{
  ConsumePrescriptionRequestPtr r = new ::Ehr::ConsumePrescriptionRequest;
  setupThirdPartyRequest(r, requestorPemFile, ownerPemFile);
  r->prescriptionId = documentToConsume;
  r->consumedPrescription = doc;
  return r;
}

/**
 * Returns a find document request
 */
FindDocumentsRequestPtr
createFindDocumentsRequest(string requestorPemFile, string ownerPemFile,
                           Ice::Long from = -1, Ice::Long till = -1,
                           DocumentType type = DOCANY, Ice::Long documentId = 0)
{
  FindDocumentsRequestPtr r = new ::Ehr::FindDocumentsRequest;
  setupThirdPartyRequest(r, requestorPemFile, ownerPemFile);
  r->hasFrom = (from != -1);
  r->from.msecs = from;
  r->hasTill = (till != -1);
  r->till.msecs = till;
  r->type = type;
  r->documentId = documentId;
  return r;
}

/**
 * Returns a list permission request
 */
ListPermissionsRequestPtr createListPermissionsRequest(string requestorPemFile, Ice::Long documentId) {
  ListPermissionsRequestPtr r = new ListPermissionsRequest;
  setupRequest(r, requestorPemFile);
  r->documentId = documentId;
  return r;
}

/**
 * Returns a set default acc request
 */
SetDefaultAccessRequestPtr createSetDefaultAccessRequest(string requestorPemFile, Ice::Long documentId,
                                                         AccessType access) {
  SetDefaultAccessRequestPtr r = new SetDefaultAccessRequest;
  setupRequest(r, requestorPemFile);
  r->documentId = documentId;
  r->access = access;
  return r;
}

/**
 * Returns a create permission request
 */
CreatePermissionRequestPtr createCreatePermissionRequest(string requestorPemFile, Ice::Long documentId,
                                                            AccountIdentifier account,
                                                            AccessType access) {
  CreatePermissionRequestPtr r = new CreatePermissionRequest;
  setupRequest(r, requestorPemFile);
  r->documentId = documentId;
  r->account = account;
  r->access = access;
  return r;
}

/**
 * Returns a remove permission request
 */
RemovePermissionRequestPtr createRemovePermissionRequest(string requestorPemFile, Ice::Long documentId,
                                                         Ice::Long permissionId) {
  RemovePermissionRequestPtr r = new RemovePermissionRequest;
  setupRequest(r, requestorPemFile);
  r->documentId = documentId;
  r->permissionId = permissionId;
  return r;
}

/**
 * Fixture that sets up an instance of ProviderI
 */
struct ProviderFixture
{
  ProviderFixture()
  {
    /* create argv inside std::vector */
    std::vector<std::string>argvvec;
    argvvec.push_back(TOSTRING(BOOST_TEST_MODULE));
    argvvec.push_back("-f" TESTCONFDIR "/" TESTCONFFILE);
    boost::optional<std::string> dbdsn;
    boost::runtime::env::get("TEST_DBDSN", dbdsn);
    if (dbdsn)
      argvvec.push_back("-odatabase.dsn=\"" + dbdsn.get() + "\"");

    /* convert to c-style const char**. this is a bit hackish */
    unsigned argc = argvvec.size();
    char **argv = new char*[argc + 2];
    for(unsigned i = 0; i < argc; ++i)
      argv[i] = strdup(argvvec[i].c_str());
    argv[argc] = NULL;

    // conf setup
    m_conf = new Configuration(argc, const_cast<char **>(argv));
    BOOST_ASSERT(m_conf->loadCertificates());

    Ice::InitializationData initdata;
    initdata.properties = Ice::createProperties();
    initdata.properties->setProperty("Ice.InitPlugins", "0"); // we do this ourself
    initdata.properties->load(m_conf->getConfFile());
    ic = Ice::initialize(initdata);

    // data setup
    m_data = new DataManager(m_conf);
    if (!*m_data || !m_data->connect())
      BOOST_FAIL("could not setup data manager");
    m_data->setCommunicator(ic);

    prov = new ProviderI(m_data);

    //empty tables
    BOOST_CHECK_NO_THROW(m_data->emptyTables())

    // get test pems
    pat1_pem_file = m_conf->getUserCert("pat1");
    pat2_pem_file = m_conf->getUserCert("pat2");
    phy1_pem_file = m_conf->getUserCert("phy1");
    phy2_pem_file = m_conf->getUserCert("phy2");
    pharm1_pem_file = m_conf->getUserCert("pharm1");
    pharm2_pem_file = m_conf->getUserCert("pharm2");

    // setup user
    m_data->createUser("pat1", Ehr::PATIENT);
    m_data->createUser("pat2", Ehr::PATIENT);
    m_data->createUser("phy1", Ehr::PHYSICAN);
    m_data->createUser("phy2", Ehr::PHYSICAN);
    m_data->createUser("pharm1", Ehr::PHARMACIST);
    m_data->createUser("pharm2", Ehr::PHARMACIST);

    // create test encrypted docs
    // nth document has every property set to a vector with 1 element: n
    for (unsigned i=0; i<NUM_TEST_DOCS; i++) {
      Ehr::EncryptedDocument doc;
      Ehr::ByteSeq data;
      data.push_back(i);
      doc.encryptedData = data;
      doc.encryptedKey = data;
      doc.initialVector = data;
      documents.push_back(doc);
    }
  }

  virtual ~ProviderFixture()
  {
    m_data->emptyTables();
    delete prov;
    delete m_data;
    delete m_conf;
  }

  Ehr::Signature createSignatureForRequest(const std::string& pemFile, ::Ehr::RequestPtr request)
  {
    // create bytestream from request
    ::Ice::ByteSeq data;
    ::Ice::OutputStreamPtr outStream = ::Ice::createOutputStream(ic);
    outStream->writeObject(request);
    outStream->writePendingObjects();
    outStream->finished(data);

    // sign it via security instance
    ::Ice::ByteSeq signatureData;
    Security::instance().sign(pemFile, data, signatureData);
    ::Ehr::Signature signature;
    signature.data = signatureData;
    return signature;
  }

  /**
   * Inserts test documents
   */
  void insertDocuments()
  {
    // 0-2: docs for pat 1
    // 3-4: docs for pat 2
    string patient = pat1_pem_file;
    for (unsigned i=0; i<NUM_TEST_DOCS; i++)
    {
      if (i == NUM_TEST_DOCS_FIRST_PAT)
        patient = pat2_pem_file;
      Ehr::CreatePrescriptionRequestPtr r = createCreatePrescriptionRequest(
          phy1_pem_file, patient, documents[i]);
      prov->createPrescription(r,
                               createSignatureForRequest(phy1_pem_file, r),
                               createSignatureForRequest(patient, r),
                               Ice::Current());
    }
  }

  /**
   * Removes all documents from db
   */
  void removeDocuments() {
    m_data->emptyTables();
  }

  /**
   * Search for documents with a req and check returned number
   */
  void checkFindDocuments(ProviderI *prov, Ice::Long expectedSize,
                          FindDocumentsRequestPtr r,
                          string requestorPemFile = "",
                          string ownerPemFile = "",
                          Ice::Current c = Ice::Current())
  {
    if (requestorPemFile.empty())
      requestorPemFile = pat1_pem_file;
    if (ownerPemFile.empty())
      ownerPemFile = pat1_pem_file;
    DocumentList l = prov->findDocuments(r,
                                         createSignatureForRequest(requestorPemFile, r),
                                         createSignatureForRequest(ownerPemFile, r),
                                         c);
    BOOST_CHECK_EQUAL(l.size(), expectedSize);
  }

  /**
   * Convenience functions for listing documents
   */
  DocumentList listDocuments(ProviderI *prov, string requestorPemFile,
      string ownerPemFile, Ice::Current c = Ice::Current())
  {
    ::Ehr::ListDocumentsRequestPtr r = createListDocumentsRequest(requestorPemFile, ownerPemFile);
    return prov->listDocuments(r,
                               createSignatureForRequest(requestorPemFile, r),
                               createSignatureForRequest(ownerPemFile, r),
                               c);
  }

  ProviderI *prov;
  Configuration *m_conf;
  DataManager *m_data;
  Ice::CommunicatorPtr ic;
  string pat1_pem_file ;
  string pat2_pem_file ;
  string phy1_pem_file ;
  string phy2_pem_file ;
  string pharm1_pem_file ;
  string pharm2_pem_file ;

  vector<Ehr::EncryptedDocument> documents; // test documents
};

#endif
